;
;	DMA Macro
;
;	dmaaddr - set the 22-bit Q-Bus DMA address and DMA direction. DMA
;	direction is bit 0 of first paramater and is assumed to be set for
;	subsequent DMA reads and cleared for writes.
;
.macro	dmaaddr
	push	r24
	ldi	r24, 0xff
	cli
	cbi	b_RD
	out	dataportdir, r24
	out	dataportout, zero
	sbi	dataportout, 2
	sbi	b_ALEW
	cbi	b_ALEW
	out	dataportout, @0
	sbi	b_WR
	cbi	b_WR
	out	dataportout, @1
	sbi	b_WR
	cbi	b_WR
	out	dataportout, @2
	sbi	b_WR
	cbi	b_WR
	sei
	pop	r24
.endmacro
;
;	dmaread - read one word from the Q-Bus via DMA to the DMA receive
;	data register in CPLD B
;
.macro	dmaread
	sbi	b_DMR			; Request DMA
	push	r24
	ldi	r24, dmatmo

dmareadloop1:
	dec	r24
	sbis	i_DMG			; Wait for DMA data transfer phase
	brne	dmareadloop1		; Skip branch to loop if DMG is set
	breq	dmareadtmo		; In case we reach here when r24 is 0.
;
	cbi	b_DMR
;;;
	cli				;;; Need to disable interrupt as we use the IO Port
	cbi	b_RD			;;;
	ldi	r24, 0xff		;;;
	out	dataportdir, r24		;;;
	ldi	r24, 4			;;;
	out	dataportout, r24	;;; Read Sequence 1 = DMA Data Register
	sbi	b_ALER			;;;
	cbi	b_ALER			;;;
	out	dataportdir, zero	;;;
	sbi	b_RD			;;;
	nop				;;;
	nop				;;;
	nop				;;;
	in	@0, dataportin		;;;
	sbi	b_ALER			;;;
	cbi	b_ALER			;;;
	nop				;;;
	nop				;;;
	nop				;;;
	in	@1, dataportin		;;;
	sbi	b_ALER			;;;
	cbi	b_ALER			;;;
	cbi	b_RD			;;;
	sei				; Enable Interrupts
;;;
	clc
	rjmp	dmareadexit
dmareadtmo:
	cbi	b_DMR
	sbi	b_ABORT
	cbi	b_ABORT
	sec
dmareadexit:
	pop	r24
.endmacro
;
;	dmawrt - write one word from the DMA transmit data register in CPLD A
;	via DMA to the Q-Bus
;
.macro	dmawrt
	push	r24
	cli
	cbi	b_RD
	ldi	r24, 0xff
	out	dataportdir, r24	;;;
	ldi	r24, 2
	out	dataportout, r24
	sbi	b_ALEW
	cbi	b_ALEW
	out	dataportout, @0		;;;
	sbi	b_WR			;;;
	cbi	b_WR			;;;
	out	dataportout, @1		;;;
	sbi	b_WR			;;;
	cbi	b_WR			;;;
	sei
	sbi	b_DMR
	ldi	r24, dmatmo
dmawriteloop2:
	dec	r24
	sbis	i_DMG
	brne	dmawriteloop2
	breq	dmawritetmo
	cbi	b_DMR
	clc
	rjmp	dmawriteexit

dmawritetmo:
	cbi	b_DMR
	sbi	b_ABORT
	cbi	b_ABORT
	sec
dmawriteexit:
	pop	r24
.endmacro