;--------------------------------------------------------------------------
;
;	2019-01-06	Added CreatePath
;
;	Combine the current working path with a new path/file name to form
;	a fully qualified path. It is assumed that both string are valid and
;	existing paths or files respectively. E.g. have been checked using
;	Name2DirEntry, i.e. both, the working path and the new path/file
;	name must exist.
;	The function will scan the new path/file for the delimiter. If the
;	substring corresponds to ".." then it will remove the top element
;	from the working path. If it is a filename it will be added to the
;	working path. 
;	2019-07-18	Paths preceeded with DELIM are now handled as absolute paths
;			Use pointer and not location Path
;
;	Input:
;	X	Pointer to the new path/file zero terminated string
;	Z	Pointer to the current working path
;
;	Output:
;	Working path updated with 
;
;	Registers (2022-05-13)
;	r17
;
CreatePath:
	push	xl
	push	xh
	push	yl
	push	yh
	push	zl
	push	zh
	movw	r1:r0, zh:zl		; Save pointer to current path
	ld	r17, X
	cpi	r17, DELIM		; Absolute Path?
	brne	CreatePath010
	st	Z, zero
	adiw	X, 1
	ld	r17, X
	tst	r17
	brne	CreatePath010
	rjmp	CreatePathfin

CreatePath010:
	rcall	CopyName
	brtc	CreatePathfin
	
	lds	r17, NameBuffer+0
	cpi	r17, '.'
	brne	CreatePath040
	lds	r17, NameBuffer+1
	cpi	r17, '.'
	brne	CreatePath040
	lds	r17, NameBuffer+2
	tst	r17
	brne	CreatePath040
;
;	Move to parent directory check if we are not at the root level.
;	If we are not at the root level and must remove the top directory
;
	movw	zh:zl, r1:r0
	movw	yh:yl, r1:r0		; Initialise empty path
CreatePath020:	
	ld	r17, Z+			; Scan Path
	tst	r17
	breq	CreatePath030		; Reached end of path
	cpi	r17, DELIM		; delimiter?
	brne	CreatePath020
	movw	Y, Z			; Remember delimiter
	sbiw	Y, 1
	rjmp	CreatePath020
;
;	Y points either to the last delimiter or to the empty Path
;
CreatePath030:
	st	Y, zero			; Terminate path
	cpi	r24, DELIM		; Another Path Element
	brne	CreatePathfin		; no so we are done
	rjmp	CreatePath010
;
;	need to add NameBuffer to Path
;
CreatePath040:
	movw	zh:zl, r1:r0
	ld	r17, Z
	tst	r17
	breq	CreatePath055		; Empty Path just add NameBuffer

CreatePath050:
	ld	r17, Z+
	tst	r17
	brne	CreatePath050
	ldi	r17, DELIM
	st	-Z, r17		; New delimiter
	adiw	Z, 1			; readjust
CreatePath055:
	ldi	yl, low(NameBuffer)
	ldi	yh, high(NameBuffer)
CreatePath060:
	ld	r17, Y+
	st	Z+, r17
	tst	r17
	brne	CreatePath060		; Copy until the end of the element
	cpi	r24, DELIM		; Another command element?
	breq	CreatePath010		; Yes process it
	rjmp	CreatePathfin		; Done
;
CreatePathfin:
	pop	zh
	pop	zl
	pop	yh
	pop	yl
	pop	xh
	pop	xl
	ret 
