;--------------------------------------------------------------------------
;
;	Match filename
;	Y		Pointer to datastructure setup to read a directory
;			entry (e.g. after calling ReadDir)
;	NameBuffer	null terminate filename
;
;	Completioncode:
;	CC		entry matches filename
;	CS		entry does not match
;
;	Registers:
;	r16, r17, r18
;
;	Alternate entry:
;
;	This entry will copy back the found real filename to the buffer which
;	pointer is stored at P_DirName of the datastructure. The name is not
;	zero terminated, so in case you need a zero terminated string you must
;	clear the buffer in advance. This feature is normally used by the 
;	Name2DirEntry function to update inline the given path with the real
;	filenames. This is because filenames are case insensitive but sometimes
;	we want to give feedback to the user with the as it is stored in the
;	directory with the correct case. One example is the "pwd" print working
;	directory command.
;
MatchFileNameUpd:
	set
	cpse	zero, zero
MatchFileName:
	clt
	push	xl
	push	xh
	push	zl
	push	zh
	ldi	xl, low(NameBuffer)
	ldi	xh, high(NameBuffer)
	ldd	r17, Y+P_Flag
	sbrs	r17, D_Long
	rjmp	MatchSFN		; Match Short File Name
	ldd	zl, Y+P_LongFileN+0
	ldd	zh, Y+P_LongFileN+1
MatchFilelLoop:
	ld	r17, X+
	ld	r16, Z+
	ucase	r17
	ucase	r16
	cp	r17, r16
	brne	MatchFileFNF
	tst	r17
	brne	MatchFilelLoop
	clc
	brtc	MatchFileFin
	ldd	zl, Y+P_LongFileN+0
	ldd	zh, Y+P_LongFileN+1
	ldd	xl, Y+P_DirName+0
	ldd	xh, Y+P_DirName+1
MatchFileUpdateL:
	ld	r17, Z+
	cp	r17, zero		; if .eq. clears carry
	breq	MatchFileFin
	st	X+, r17
	rjmp	MatchFileUpdateL
MatchFileFNF:				; Match File-not-found
	sec
MatchFileFin:
	pop	zh
	pop	zl
	pop	xh
	pop	xl
	ret
;
;	Match short filename 
;
MatchSFN:
	ldd	zl, Y+P_DirPointer+0
	ldd	zh, Y+P_DirPointer+1
	ldi	r18, 8		; 8 character name
MatchSFNName:
	ld	r17, X+			; Get next input filename character
	ldd	r16, Z+D_Name		; Get next directory filename character
	adiw	Z, 1			; Adjust pointer
	cpi	r16, 0x20		; blank
	breq	MatchSFNDot		; Then next might be a dot
	ucase	r17			; Convert input character
	cp	r16, r17		; EQ?
	brne	MatchSFNFNF		; file not found
	dec	r18			; Next name character
	brne	MatchSFNName		;
	ld	r17, X+
MatchSFNDot:
	ldd	zl, Y+P_DirPointer+0
	ldd	zh, Y+P_DirPointer+1
;	
;	The Name part matches the input filename. Now we have various
;	possibilities
;
;	First byte of extension is a space and we have reached end of file ->match
;	First byte of extension is not a space and we have a .->possible match
;
;
	ldd	r16, Z+D_Ext		;
	cpi	r16, 0x20		; no extension?
	breq	MatchSFNNull		; then we must have reach end if input file
	cpi	r17, '.'
	brne	MatchSFNFNF		; no match
	ldi	r18, 3		; Compare extension
MatchSFNExt:
	ld	r17, X+
	ldd	r16, Z+D_Ext
	adiw	Z, 1
	cpi	r16, 0x20		; blank
	breq	MatchSFNNull		; Then must be end of input filename
	ucase	r17
	cp	r16, r17
	brne	MatchSFNFNF	
	dec	r18
	brne	MatchSFNExt
	ld	r17, X+
MatchSFNNull:
	tst	r17
	brne	MatchSFNFNF
MatchSFNFound:
	clc
	brtc	MatchSFNFin
	ldd	zl, Y+P_DirPointer+0
	ldd	zh, Y+P_DirPointer+1
	ldd	xl, Y+P_DirName+0
	ldd	xh, Y+P_DirName+1

	ldi	r18, 8
MatchSFNFound010:
	ld	r16, Z+
	cpi	r16, ' '
	breq	MatchSFNFound020
	st	X+, r16
	dec	r18
	brne	MatchSFNFound010

MatchSFNFound020:
	ld	r16, Z+
	cpi	r16, ' '
	breq	MatchSFNFound040
	ldi	r17, '.'
	st	X+, r17
	st	X+, r16

	ld	r16, Z+
	cpi	r16, ' '
	breq	MatchSFNFound040
	st	X+, r16

	ld	r16, Z+
	cpi	r16, ' '
	breq	MatchSFNFound040
	st	X+, r16

MatchSFNFound040:
	clc
	rjmp	MatchSFNFin
MatchSFNFNF:
	sec
MatchSFNFin:
	pop	zh
	pop	zl
	pop	xh
	pop	xl
	ret
 
