;--------------------------------------------------------------------------
;
;	Name2DirEntry
;
;	This function takes a pointer to a path name. It walks through the 
;	path and checks if the name exists in the current directory.
;	-	If the name is not found it will return an error. 
;	-	If the name found is a normal filename and the name is not the
;		last name in the path it will return an error. Else it will 
;		return success
;	-	If the name found is a directory it will take the next name and 
;		search for it in the new directory.
;	-	If the end of the path has been reached it will return success
;
;	Input:
;	Y		Pointer to data structure for directory IO
;	X		Pointer to zero terminated path
;
;	Output:
;	Directory structure updated 
;	Y+P_StartCluster	start cluster of file/directory
;	Y+P_DirPointer		points to the directory entry
;	r24			the terminating character
;
;	Conditions:
;	CS		file not found
;	CC		file found
;
;	Registers
;	r17
;
;******	20220-05-13	New register usage, list of caller:
;******			readinit		ok
;******			cmdcd			ok
;******			cmddir			ok			
;******			attachcmd		ok
;******			cmdtype			ok
;******			cmdromload		ok
;******			
Name2DirEntry:
	push	zl
	push	zh
Name2DirEntry000:

	sts	pprint+0, xl
	sts	pprint+1, xh
	std	Y+P_DirName+0, xl
	std	Y+P_DirName+1, xh
	rcall	CopyName		; Copy a file/directory name
	brtc	Name2DirEntryfin	; Rest of name is empty -> done
	
	rcall	Name2ChkRoot		; Skip .. in path when we are
	brcs	Name2DirEntry020	; at root

	ldd	r17, Y+P_StartClust+0	; Get current working directory 
	std	Y+P_Cluster+0, r17	; start cluster
	ldd	r17, Y+P_StartClust+1
	std	Y+P_Cluster+1, r17
	ldd	r17, Y+P_StartClust+2
	std	Y+P_Cluster+2, r17
	ldd	r17, Y+P_StartClust+3
	std	Y+P_Cluster+3, r17
	rcall	OpenDir			; Open the directory
Name2DirEntry010:
	rcall	ReadDir			; Read Directory Entry
	brcs	Name2DirEntryfnf	; End of Directory reached
	rcall	MatchFileNameUpd	; Compare Entry with Name
	brcs	Name2DirEntry010	; Not this one
	ldd	zl, Y+P_DirPointer+0	; Directory found
	ldd	zh, Y+P_DirPointer+1
	ldd	r17, Z+D_Attr     
	sbrs	r17, A_Directory
	rjmp	Name2DirEntrynad	; this is not a directory
	ldd	r17, Z+D_Cluster+0	; Copy Cluster to start 
	std	Y+P_StartClust+0, r17
	ldd	r17, Z+D_Cluster+1
	std	Y+P_StartClust+1, r17
	std	Y+P_StartClust+2, zero
	std	Y+P_StartClust+3, zero
	lds	r17, VolStatus
	sbrs	r17, FAT32
	rjmp	Name2DirEntry020
	ldd	r17, Z+D_ClusterH+0
	std	Y+P_StartClust+2, r17
	ldd	r17, Z+D_ClusterH+1
	std	Y+P_StartClust+3, r17
Name2DirEntry020:
	cpi	r24, DELIM		; Is there potentially another
	breq	Name2DirEntry000	; name then proceed with next name
Name2DirEntryfin:
	clc
	pop	zh
	pop	zl
	ret
;
;	Current name is not a directory
;
Name2DirEntrynad:
	cpi	r24, CR
	breq	Name2DirEntryfin
	tst	r24
	breq	Name2DirEntryfin
;
;
;
Name2DirEntryfnf:
	sec
	pop	zh
	pop	zl
	ret
;
;	We cannot do a step to '..' if we are at the root
;
Name2ChkRoot:
	ldd	r17, Y+P_StartClust+0
	tst	r17
	brne	Name2ChkRoot010		; not at root directory
	ldd	r17, Y+P_StartClust+1
	tst	r17
	brne	Name2ChkRoot010		; not at root directory
	ldd	r17, Y+P_StartClust+2
	tst	r17
	brne	Name2ChkRoot010		; not at root directory
	ldd	r17, Y+P_StartClust+3
	tst	r17
	brne	Name2ChkRoot010		; not at root directory
	lds	r17, NameBuffer+0
	cpi	r17, '.'
	brne	Name2ChkRoot010		; not a '..' step
	lds	r17, NameBuffer+1
	cpi	r17, '.'
	brne	Name2ChkRoot010		; not a '..' step
	lds	r17, NameBuffer+2
	tst	r17
	brne	Name2ChkRoot010		; not a '..' step even if it starts with ..
	sec				; at root and step is '..'
	ret

Name2ChkRoot010:
	clc
	ret
 
