;--------------------------------------------------------------------------
;
;	Read One Byte from open file
;
;	Get next byte from file. 
;
;	-	Return 0x1a (^Z for end-of-file) if already all bytes read
;	-	Update file pointer
;	-	read next sector in cluster if last byte of sector already read
;	-	link to next cluster if all bytes in cluster already read
;
;	Input
;	
;	Y	file control block
;
;	Output
;
;	CS	Error
;	r24	Error Code, most important 0x1a for end-of-file
;
;	CC	Error
;	r24	Byte
;
;	Registers
;	
;	r17
;	
ReadFileByte:
;
;	First check if we already reached end-of-file
;

	push	r4
	push	r5
	push	r6
	push	r7

	ldd	r4, Y+fcb_position+0	; get current posistion
	ldd	r5, Y+fcb_position+1
	ldd	r6, Y+fcb_position+2
	ldd	r7, Y+fcb_position+3	

	ldd	r17, Y+fcb_filesize+0	; compare to file size
	cp	r4, r17
	ldd	r17, Y+fcb_filesize+1
	cpc	r5, r17
	ldd	r17, Y+fcb_filesize+2
	cpc	r6, r17
	ldd	r17, Y+fcb_filesize+3
	cpc	r7, r17
	brlo	ReadFileByte010		; we still have some bytes left
	
	ldi	r24, 0x1F & 'Z'	; ^Z	End-of-File

	pop	r7
	pop	r6
	pop	r5
	pop	r4

	sec
	ret
	
ReadFileByte010:
;
;	Next check if there is a byte left in the sector buffer
;
	ldi	r17, 1
	add	r4, r17			; increment position
	adc	r5, zero
	adc	r6, zero
	adc	r7, zero
	std	Y+fcb_position+0, r4
	std	Y+fcb_position+1, r5
	std	Y+fcb_position+2, r6
	std	Y+fcb_position+3, r7
	
	ldd	r4, Y+fcb_byteinsec+0	; get offset in sector
	ldd	r5, Y+fcb_byteinsec+1

	ldi	r17, low(512)		; reached end of sector
	cp	r4, r17
	ldi	r17, high(512)
	cpc	r5, r17
	brlo	ReadFileByte030			; no still bytes in sector
;
;	Need a new sector, as ReadFileOpen already has read the first sector
;	we need to increment the sector number first
;
	ldd	r17, Y+fcb_sector+0	; increment sector for next in cluster
;	add	r17, one
	subi	r17, byte1(-1)
	std	Y+fcb_sector+0, r17
	ldd	r17, Y+fcb_sector+1
;	adc	r17, zero
	sbci	r17, byte2(-1)
	std	Y+fcb_sector+1, r17
	ldd	r17, Y+fcb_sector+2
;	adc	r17, zero
	sbci	r17, byte3(-1)
	std	Y+fcb_sector+2, r17
	ldd	r17, Y+fcb_sector+3
;	adc	r17, zero
	sbci	r17, byte4(-1)
	std	Y+fcb_sector+3, r17

	ldd	r4, Y+fcb_secinclst	; how many sectors did we already read
	lds	r17, sectperclst	; compare to number of sectors in cluster
	cp	r4, r17
	brlo	ReadFileByte020		; still within current cluster
;
;	Need next cluster
;
	rcall	LinkedCluster		; get linked cluster
	rcall	Cluster2Sector		; convert cluster to sector
	clr	r4
ReadFileByte020:
	inc	r4
	std	Y+fcb_secinclst, r4	; initialise number of sectors read

	ldd	r4, Y+fcb_address+0	; get buffer address
	ldd	r5, Y+fcb_address+1
	push	r4			; and save
	push	r5
	ldi	r17, led_time
	sts	led_oneshot, r17
	sbi	b_LED
	call	SD_CARD_READ
	pop	r5
	pop	r4
	brcc	ReadFileByte025
	ldi	r24, 0x80

	pop	r7
	pop	r6
	pop	r5
	pop	r4

	ret
	
ReadFileByte025:
	std	Y+fcb_address+0, r4	; restore buffer address
	std	Y+fcb_address+1, r5
	clr	r4			; initialise offset into buffer
	clr	r5
ReadFileByte030:
	push	xl
	push	xh
	ldd	xl, Y+fcb_address+0	; get buffer addres
	ldd	xh, Y+fcb_address+1
	add	xl, r4			; add offset into sector
	adc	xh, r5
	ld	r24, X			; get byte
	movw	xh:xl, r5:r4
	adiw	xh:xl, 1
	std	Y+fcb_byteinsec+0, xl	; save new offset
	std	Y+fcb_byteinsec+1, xh
	pop	xh
	pop	xl

	pop	r7
	pop	r6
	pop	r5
	pop	r4

	clc
	ret
 
 
