;--------------------------------------------------------------------------
;
;	Open File for Read access
;
;	Create a file control block with IO buffer, initialise the file
;	control block and read first sector/block
;
;	
;	Input
;
;	Z	Pointer to directory entry
;
;	Output
;
;	CS	Error
;	r24	Error Code
;
;	CC	Success
;	Y	Pointer to file control block
;
;	fcb_cluster
;	fcb_sector
;	fcb_address
;	fcb_secinclst
;	fcb_position
;	fcb_fileszie
;	fcb_byteinsec
;
;	Registers
;
;	r17, r24, r25
;
ReadFileOpen:
	ldi	r24, low(fcb_length)	; Allocate memory for a file control block
	ldi	r25, high(fcb_length)
	call	malloc
	brcc	ReadFileOpen010
	ldi	r24, 0x80
	ret

ReadFileOpen010:
	movw	Y, r25:r24		; Save to pointer
	ldi	r24, low(512)		; Allocate memory for read/write buffer
	ldi	r25, high(512)
	call	malloc
	brcc	ReadFileOpen020
	movw	r25:r24, Y
	call	free
	sec
	ldi	r24, 0x81
	ret

ReadFileOpen020:
	std	Y+fcb_address+0, r24	; Save buffer address to fcb
	std	Y+fcb_address+1, r25
	push	r24
	push	r25
	std	Y+fcb_position+0, zero	; Reset file position
	std	Y+fcb_position+1, zero
	std	Y+fcb_position+2, zero
	std	Y+fcb_position+3, zero
	
	std	Y+fcb_byteinsec+0, zero	; Reset sector position
	std	Y+fcb_byteinsec+1, zero

	ldi	r17, 1
	std	Y+fcb_secinclst+0, r17	; First sector in cluster read

	ldd	r17, Z+D_Size+0	; Get total file size to fcb
	std	Y+fcb_filesize+0, r17
	ldd	r17, Z+D_Size+1
	std	Y+fcb_filesize+1, r17
	ldd	r17, Z+D_Size+2
	std	Y+fcb_filesize+2, r17
	ldd	r17, Z+D_Size+3
	std	Y+fcb_filesize+3, r17

	std	Y+fcb_cluster+3, zero	; Initialise higher 16-bits of start cluster
	std	Y+fcb_cluster+2, zero
	lds	r17, VolStatus
	sbrs	r17, FAT32		; 
	rjmp	ReadFileOpen030			
	ldd	r17, Z+D_ClusterH+1	; for FAT-32 get higher 16-bits
	std	Y+fcb_cluster+3, r17
	ldd	r17, Z+D_ClusterH+0
	std	Y+fcb_cluster+2, r17
ReadFileOpen030:
	ldd	r17, Z+D_Cluster+1	; get lower 16-bits of start cluster
	std	Y+fcb_cluster+1, r17
	ldd	r17, Z+D_Cluster+0
	std	Y+fcb_cluster+0, r17
	rcall	Cluster2Sector		; Convert cluster to sector of first 
	call	SD_CARD_READ		; read the sector
	pop	r25			; restore buffer address
	pop	r24
	std	Y+fcb_address+0, r24
	std	Y+fcb_address+1, r25

	ldi	r17, (1<<F_Readonly) | (1<<F_Sequential)
	std	Y+fcb_flag, r17

	clc
	ret
	 
