;==========================================================================
;
;	Parsing a command line:
;
;	Command lines consist of keywords, strings, etc.. Keywords can be
;	abreviated and are case insensitive. Keywords may have a suffix
;	in the form of a number or a number followed by a colon. The parser
;	uses tables with 3 entries
;
;		-	pointer to desciption of syntax element, 0 means no further 
;			checks possible, i.e. the scan of the command failed or in
;			other words this is not a valid command
;		-	action routine to call in case of success, 0 no action routine
;			this is typically required to make note of the results produced
;			when scanning the syntax element, e.g. the suffix number, the 
;			number or the string/quoted string found
;		-	if the element matches then the third entry gives the address
;			of the next parser table or "cmd_done" in case the end of the
;			command has been reached, if no match the parser proceeds
;			to the following entry
;
;	A syntax element consists of a type followd by additional parameters. In
;	case of keywords the name of the keyword is given the mandatory part must
;	be given in capital and the optional part in lower case. In case of a sub-
;	command the parameter is a pointer to a command table (the parser calls 
;	itself using this pointer as the table pointer)
;
;	A Table must be terminated with a 0 to indicated the end. If you want
;	a command to be valid you must typically scan up to the terminating 
;	character, in our case <CR>, using the eol syntax element and a next
;	table pointer of cmd_done ( -1). You can use cmd_done with every syntax
; 	element type. 
;
;	When calling the parser Z must point to the start table (caution it must
;	be a byte pointer for lpm, typically twice the address) and X must point
;	to the string to be parsed. If the sring could be parsed successfully
;	carry will be cleared on return. Else carry will be set. X will point
;	to the position up to which the parser has analyzed the string (not
;	necessarily the end of the string)
;
commandlist:
;
;		Element		Action		Next
;
	.dw	kw_cd,		0,		cmd_cd
	.dw	kw_show,	0,		cmd_show
	.dw	kw_dir,		a_dirinit,	cmd_dir
	.dw	kw_type,	0,		cmd_type
	.dw	kw_pwd,		0,		cmd_pwd
	.dw	kw_fdisk,	0,		cmd_fdisk
	.dw	kw_detach,	0,		cmd_detach
	.dw	kw_mon, 	a_mon,		cmd_done
	.dw	kw_help,	0, 		cmd_help
	.dw	kw_activate,	0,		cmd_activate
	.dw	kw_initialise,	0,		cmd_initialise
	.dw	kw_logging,	0,		cmd_logging
	.dw	kw_no,		0, 		cmd_no
	.dw	kw_nbr, 	0, 		cmd_nbr
	.dw	kw_load,	0,		cmd_load
	.dw	kw_readinit,	0,		cmd_readinit
	.dw	kw_dumpblock,	a_dumpinit,	cmd_dumpblock
	.dw	kw_dmatest,	0,		cmd_dmatest
	.dw	kw_dmaaddr,	0,		cmd_dmaaddr
	.dw	kw_dmaread,	0,		cmd_dmaread
	.dw	kw_dmawrite,	0,		cmd_dmawrite
	.dw	kw_linked,	0,		cmd_linked
	.dw	kw_free,	0,		cmd_free
	.dw	kw_bootrom,	0,		cmd_bootrom
	.dw	kw_mount,	0,		cmd_mount
	.dw	kw_readchk,	a_readchkinit,	cmd_readchk
	
comandliststartup:
	.dw	kw_attach, 	0, 		cmd_attach	;
commandlistrlv12:
	.dw	kw_rlv12,	0,		cmd_init	; 
	.dw	0
;
;
;
cmd_readchk:
	.dw	kw_number,	a_readchkblock,	cmd_readchk2
	.dw	0
cmd_readchk2:
	.dw	kw_number,	a_readchksize,	cmd_readchk3
	.dw	kw_eol, 	readchk,	cmd_done
	.dw	0
cmd_readchk3:
	.dw	kw_number,	a_readchkaddr,	cmd_readchk4
	.dw	kw_eol,		readchk,	cmd_done
cmd_readchk4:
	.dw	kw_eol,		readchk,	cmd_done
	.dw	0
	
cmd_free:
	.dw	kw_eol,		showfree,	cmd_done
	.dw	0
	
cmd_linked:
	.dw	kw_number, 	a_savnbr, 	cmd_linked1
	.dw	0
cmd_linked1:
	.dw	kw_eol,		showlinked,	cmd_done
	.dw	0

cmd_init:
	.dw	kw_eol,		a_init,		cmd_done	;
	.dw	0

cmd_dmawrite:
	.dw	kw_number,	a_savnbr,	cmd_dmawrite1
	.dw	0
cmd_dmawrite1:
	.dw	kw_eol,		cmddmawrite,	cmd_done
	.dw	0

cmd_dmaaddr:
	.dw	kw_number,	a_savnbr,	cmd_dmaaddr1
	.dw	0
cmd_dmaaddr1:
	.dw	kw_eol,		cmddmaaddr,	cmd_done
	.dw	0

cmd_dmaread:
	.dw	kw_eol,		cmddmaread,	cmd_done
	.dw	0
;
;	dmatest <address> <pattern>
;
cmd_dmatest:
	.dw	kw_number,	a_savnbr,	cmd_dmatest1
	.dw	0
cmd_dmatest1:
	.dw	kw_number, 	a_pattern,	cmd_dmatest2
	.dw	kw_eol,		dmatest,	cmd_done
	
	.dw	0
cmd_dmatest2:
	.dw	kw_eol,		dmatest,	cmd_done
	.dw	0

cmd_readinit:
	.dw	kw_eol,		readinit, 	cmd_done
	.dw	0

cmd_nbr:
	.dw	kw_number, 	a_savnbr,	cmd_nbr1
	.dw	0
cmd_nbr1:
	.dw	kw_eol,		prtnbr,		cmd_done
	.dw	0

cmd_load:
	.dw	kw_number, 	a_savnbr,	cmd_load1
;	.dw	kw_string, 	a_cd,		cmd_bootromload1
;	.dw	kw_qstring, 	a_cd,		cmd_bootromload1
	.dw	0

cmd_load1:
	.dw	kw_eol,		loadboot,	cmd_done
	.dw	0

cmd_fdisk:
	.dw	kw_eol,		fdisk,		cmd_done
	.dw	0


;
;	"no"
;
cmd_no:
	.dw	kw_logging, 	a_setno,	cmd_logging
	.dw	kw_bootrom,	a_setno,	cmd_bootrom
	.dw	0
;
;	rom
;
cmd_bootrom:
	.dw	kw_eol,		cmdrom,		cmd_done
	.dw	kw_rw,		cmdromrw,	cmd_bootromrw
	.dw	kw_ro,		cmdromro,	cmd_bootromro
	.dw	kw_reset,	0,		cmd_bootromreset
	.dw	kw_load,	0,		cmd_bootromload
	.dw	0

cmd_bootromload:
	.dw	kw_qstring,	a_cd,		cmd_bootromload1
	.dw	kw_string,	a_cd,		cmd_bootromload1
	.dw	0

cmd_bootromload1:
	.dw	kw_eol,		cmdromload,	cmd_done
	.dw	0
	
cmd_bootromrw:
	.dw	kw_eol,		cmdromrw,	cmd_done
	.dw	0
	
cmd_bootromro:
	.dw	kw_eol,		cmdromro,	cmd_done
	.dw	0
	
cmd_bootromreset:
	.dw	kw_eol,		cmdromreset,	cmd_done
	.dw	0

;
;	logg
;
cmd_logging:
	.dw	kw_register,	0,		cmd_logregister
	.dw	kw_interrupt,	0,		cmd_loginterrupt
	.dw	kw_units,	0,		cmd_logunits
	.dw	kw_pbn,		0,		cmd_logpbn
	.dw	kw_rom,		0,		cmd_logrom
	.dw	kw_attdev, 	a_dev,		cmd_logunit
	.dw	kw_eol,		logstatus,	cmd_done
	.dw	0

cmd_logrom:
	.dw	kw_eol,		logrom,		cmd_done
	.dw	0
	
cmd_logpbn:
	.dw	kw_eol,		logpbn, 	cmd_done
	.dw	0

cmd_logregister:
	.dw	kw_eol,		logreg, 	cmd_done
	.dw	0

cmd_loginterrupt:
	.dw	kw_eol,		logiack, 	cmd_done
	.dw	0

cmd_logunits:
	.dw	kw_eol,		logunits,	cmd_done
	.dw	0
	
cmd_logunit:
	.dw	kw_eol,		logunit,	cmd_done
	.dw	0
	
cmd_activate:
	.dw	kw_attpar, 	a_par,		cmd_activate1
	.dw	0

cmd_activate1:
	.dw	kw_eol,		activpar,	cmd_done
	.dw	0
	
cmd_initialise:
	.dw	kw_attpar,	a_par,		cmd_initialise1
	.dw	0

cmd_initialise1:
	.dw	kw_eol,		initpar,	cmd_done
	.dw	0

cmd_attach:
	.dw	kw_attdev,	a_dev,		cmd_attach0
	.dw	0

cmd_attach0:
	.dw	kw_attpar,	a_par, 		cmd_attach2
	.dw	kw_qstring,	a_cd,		cmd_attach1
	.dw	kw_string,	a_cd,		cmd_attach1
	.dw	0
	
cmd_attach1:
	.dw	kw_eol,		attachcmd,	cmd_done
	.dw	0

cmd_attach2:
	.dw	kw_eol,		attachpar,	cmd_done
	.dw	0

cmd_detach:
	.dw	kw_attdev,	a_dev,		cmd_detach1
	.dw	0

cmd_detach1:
	.dw	kw_eol,		detachunit,	cmd_done
	.dw	0

cmd_pwd:
	.dw	kw_eol,		pwdcmd,		cmd_done
	.dw	0
	
cmd_type:
	.dw	kw_qstring,	a_cd,		cmd_type1
	.dw	kw_string,	a_cd,		cmd_type1
	.dw	0

cmd_type1:
	.dw	kw_eol,		cmdtype,	cmd_done
	.dw	0

cmd_dir:
	.dw	kw_eol,		cmddir,		cmd_done
	.dw	sw_dira, 	a_dirswa, 	cmd_dir
	.dw	kw_qstring,	a_cd,		cmd_dir1
	.dw	kw_string,	a_cd,		cmd_dir1
	.dw	0
	
cmd_dir1:
	.dw	kw_eol,		cmddirdir,	cmd_done
	.dw	0

cmd_cd:	
	.dw	kw_qstring,	a_cd,		cmd_cd1
	.dw	kw_string,	a_cd,		cmd_cd1
	.dw	kw_eol,		cmdcdroot,	cmd_done
	.dw	0
	
cmd_cd1:
	.dw	kw_eol,		cmdcd,		cmd_done
	.dw	0

;
;	show
;	
cmd_show:
	.dw	kw_logging,	logprint,	cmd_done
	.dw	kw_showunits,	cmdshowunits,	cmd_done
	.dw	kw_showpart,	cmdshowpart,	cmd_done
	.dw	kw_showvers,	cmdshowvers,	cmd_done
	.dw	kw_showstatus, 	cmdshowstatus,	cmd_done
	.dw	kw_bootrom,	cmdshowrom,	cmd_done
	.dw	0
	
cmd_help:
	.dw	kw_eol,		cmdhelp,	cmd_done
	.dw	0

cmd_dumpblock:
	.dw	sw_dumpr,	a_dumpswitchr,	cmd_dumpblock		; optional switch
	.dw	sw_dumph,	a_dumpswitchh,	cmd_dumpblock
	.dw	sw_dumpc,	a_dumpswitchc, 	cmd_dumpblock
	.dw	sw_dumpb,	a_dumpswitchb,	cmd_dumpnbr
	.dw	kw_dumpdev,	a_dumpdevice,	cmd_dumpblock1
	.dw	0
	
cmd_dumpnbr:
	.dw	kw_number,	a_dumpblknbr,	cmd_dumpblock
	.dw	kw_lambda,	0,		cmd_dumpblock
	.dw	0

cmd_dumpblock1:
	.dw	kw_eol,		cmddumpbl,	cmd_done
	.dw	0

cmd_mount:
	.dw	kw_eol,		MountVolume,	cmd_done
	.dw	0

kw_no:		P_KEYWORD	"NO"
kw_register:	P_KEYWORD	"REGister"
kw_interrupt:	P_KEYWORD	"INTerrupt"
kw_string:	P_STRING			; string, anything except white spaces
kw_eol:		P_EOL
kw_qstring:	P_QSTRING
kw_name:	P_NAME				; Name must start with a-z, $, @ and may contain 0-9, ., -, _
kw_number:	P_NUMBER			; Number
kw_lambda:	P_LAMBDA
;
kw_rlv12:	P_KEYWORD	"[RLV12]"
kw_type:	P_KEYWORD	"TYpe"
kw_linked:	P_KEYWORD	"LINKed"
kw_cd:		P_KEYWORD	"CD"		; Keyword
kw_units:	P_KEYWORD	"UNIts"
kw_pbn:		P_KEYWORD	"PBN"
kw_show:	P_KEYWORD	"Show"
kw_dir:		P_KEYWORD	"DIR"
kw_pwd:		P_KEYWORD	"PWD"
kw_attach:	P_KEYWORD	"ATTach"
kw_attdev:	P_KEYSUFF	"RL"
kw_attpar:	P_KEYSUFF	"Partition"
kw_detach:	P_KEYWORD	"DEtach"
kw_mount:	P_KEYWORD	"MOUnt"
kw_showunits:	P_KEYWORD	"Units"
kw_showpart:	P_KEYWORD	"Partitions"
kw_showvers:	P_KEYWORD	"Version"
kw_showstatus:	P_KEYWORD	"STAtus"
kw_activate:	P_KEYWORD	"ACTivate"
kw_initialise:	P_KEYWORD	"INITialise"
kw_logging:	P_KEYWORD	"LOGging"
kw_mon:		P_CHAR		'$'
kw_help:	P_KEYWORD	"HElp"
kw_fdisk:	P_KEYWORD	"FDISK"
kw_nbr:		P_KEYWORD	"Number"
kw_load:	P_KEYWORD	"Load"
kw_readinit:	P_KEYWORD	"READINIT"
kw_dumpdev:	P_KEYSUFF	"RL"
kw_dumpblock:	P_KEYWORD	"DUMP"
kw_dmaread:	P_KEYWORD	"DMARead"
kw_dmawrite:	P_KEYWORD	"DMAWrite"
kw_dmaaddr:	P_KEYWORD	"DMAAddress"
kw_dmatest:	P_KEYWORD	"DMATest"
kw_bootrom:	P_KEYWORD	"BOOTROM"
kw_rom:		P_KEYWORD	"ROM"
kw_rw:		P_KEYWORD	"RW"
kw_ro:		P_KEYWORD	"RO"
kw_reset:	P_KEYWORD	"RESet"
kw_free:	P_KEYWORD	"FREE"
kw_readchk:	P_KEYWORD	"READcheck"

sw_dira:	P_SWITCH	"a"
sw_dumph:	P_SWITCH	"h"
sw_dumpr:	P_SWITCH	"r"
sw_dumpc:	P_SWITCH	"c"
sw_dumpb:	P_SWITCH	"b"
