;--------------------------------------------------------------------------
;
;	fdisk
;
;--------------------------------------------------------------------------
;
fdisk:
	ldi	yl, low(pcluster)	; Prepare an IO control block
	ldi	yh, high(pcluster)	; 
	ldi	zl, low(pbuffer)	; set IO buffer address
	ldi	zh, high(pbuffer)	; 
	std	Y+P_Address, zl
	std	Y+P_Address+1, zh

	std	Y+P_Sector+0, zero	; Set sector to MBR
	std	Y+P_Sector+1, zero
	std	Y+P_Sector+2, zero
	std	Y+P_Sector+3, zero

	sts	pdirstartc+0, zero
	sts	pdirstartc+1, zero
	sts	pdirstartc+2, zero
	sts	pdirstartc+3, zero
	
	sts	pextended+0, zero
	sts	pextended+1, zero
	sts	pextended+2, zero
	sts	pextended+3, zero
	
	std	Y+P_Flag, zero		; No partition seen

fdisknext:
	ldd	r16, Y+P_Sector+0
	sts	pprint+2, r16
	ldd	r16, Y+P_Sector+1
	sts	pprint+3, r16
	ldd	r16, Y+P_Sector+2
	sts	pprint+4, r16
	ldd	r16, Y+P_Sector+3
	sts	pprint+5, r16
	call	SD_CARD_READ
	brcs	fdiskerr81

	lds	r16, pbuffer+510
	sts	pprint+0, r16
	cpi	r16, 0x55
	brne	fdiskerr82
	lds	r16, pbuffer+511
	sts	pprint+1, r16
	cpi	r16, 0xAA
	breq	fdiskanalyze
;
;
;
fdiskerr82:
	call	print
	.db	"fdisk -- invalid signature 0x", 0x80, 0x81, " in sector 0x", 0x85, 0x84, 0x83, 0x82, CR, LF, 0, 0
	ldi	r24, 0x82
	rjmp	fdiskerr
fdiskerr81:
	ldi	r24, 0x81
fdiskerr:
	sec
	ret				; Exit with CS

fdiskanalyze:

	call	print
	.db	"Sector: 0x", 0x85, 0x84, 0x83, 0x82, ", Signature: 0x", 0x81, 0x80, " ", CR, LF
	.db	"         Starting       Ending", CR, LF
	.db	"#: id  cyl  hd sec -  cyl  hd sec [     start -       size] ", CR, LF
	.db	"------------------------------------------------------------------------ ", CR, LF, 0

	ldd	zl, Y+P_Address+0
	ldd	zh, Y+P_Address+1	
	subi	zl, low(-M_PartTable)	
	sbci	zh, high(-M_PartTable)	
	ldi	r18, 4
	ldd	r16, Y+P_Flag
	cbr	r16, (1<<Part_Next)
	std	Y+P_Flag, r16
fdiskanalyzenext:
	rcall	fdiskprintentry
	rcall	fdiskcheckextended
	adiw	Z, M_PartEntry
	dec	r18
	brne	fdiskanalyzenext
	
	ldd	r16, Y+P_Flag
	sbrs	r16, Part_Next
	rjmp	fdiskanalyzedone
	cbr	r16, (1<<Part_Next)
	std	Y+P_Flag, r16

	ldd	r16, Y+P_Cluster+0
	std	Y+P_Sector+0, r16		
	ldd	r16, Y+P_Cluster+1
	std	Y+P_Sector+1, r16
	ldd	r16, Y+P_Cluster+2
	std	Y+P_Sector+2, r16
	ldd	r16, Y+P_Cluster+3
	std	Y+P_Sector+3, r16
	rjmp	fdisknext

fdiskanalyzedone:
	clc
	ret

fdiskcheckextended:
	ldd	r16, Z+M_PartType
	cpi	r16, 0x05
	breq	fdiskcheckextended010
	ret
fdiskcheckextended010:
	ldd	r16, Y+P_Flag
	sbrc	r16, Part_Ext
	rjmp	fdiskcheckextended020
	sbr	r16, (1<<Part_Ext | 1<<Part_Next)
	std	Y+P_Flag, r16

	ldd	r16, Z+M_PartStart+0
	std	Y+P_Cluster+0, r16	; Remember the extended parititon offset
	sts	pextended+0, r16	; Remember the global extended parititon offset

	ldd	r16, Z+M_PartStart+1
	std	Y+P_Cluster+1, r16
	sts	pextended+1, r16

	ldd	r16, Z+M_PartStart+2
	std	Y+P_Cluster+2, r16
	sts	pextended+2, r16
	
	ldd	r16, Z+M_PartStart+3
	std	Y+P_Cluster+3, r16
	sts	pextended+3, r16
	ret

fdiskcheckextended020:
	sbr	r16, (1<<Part_Next)
	std	Y+P_Flag, r16
	
	lds	r0, pextended+0
	ldd	r16, Z+M_PartStart+0
	add	r16, r0
	std	Y+P_Cluster+0, r16	

	lds	r0, pextended+1
	ldd	r16, Z+M_PartStart+1
	adc	r16, r0
	std	Y+P_Cluster+1, r16	

	lds	r0, pextended+2
	ldd	r16, Z+M_PartStart+2
	adc	r16, r0
	std	Y+P_Cluster+2, r16	

	lds	r0, pextended+3
	ldd	r16, Z+M_PartStart+3
	adc	r16, r0
	std	Y+P_Cluster+3, r16	

	ret

fdiskprintentry:
	ldi	r16, '5'
	sub	r16, r18
	sts	pprint+0, r16
	ldd	r16, Z+M_PartType
	sts	pprint+1, r16

	ldd	r16, Z+M_PartStart+0
	sts	pprint+2, r16
	ldd	r16, Z+M_PartStart+1
	sts	pprint+3, r16
	ldd	r16, Z+M_PartStart+2
	sts	pprint+4, r16
	ldd	r16, Z+M_PartStart+3
	sts	pprint+5, r16

	ldd	r16, Z+M_PartType
	tst	r16
	brne	fdiskprintentry110
;
;	Empty Partition Entry
;
	call	print	
	.db	0x90, ": 00    0   0   0 -    0   0   0 [         0 -          0] unused"
	.db	CR, LF, 0x00, 0x00
	ret

fdiskprintentry110:
	cpi	r16, 0x05
	breq	fdiskprintentry120
;
;	Normal Partition Start: Y+P_Sector + Z+M_PartStart
;
	lds	r16, pprint+2
	ldd	r0, Y+P_Sector+0
	add	r16, r0
	sts	pprint+2, r16
	lds	r16, pprint+3
	ldd	r0, Y+P_Sector+1
	adc	r16, r0
	sts	pprint+3, r16
	lds	r16, pprint+4
	ldd	r0, Y+P_Sector+2
	adc	r16, r0
	sts	pprint+4, r16
	lds	r16, pprint+5
	ldd	r0, Y+P_Sector+3
	adc	r16, r0
	sts	pprint+5, r16
	rjmp	fdiskprintentry130
;
;	Extended Partition Start: pextended + Z+__PartStart
;
fdiskprintentry120:
	lds	r16, pprint+2
	lds	r0, pextended+0
	add	r16, r0
	sts	pprint+2, r16
	lds	r16, pprint+3
	lds	r0, pextended+1
	adc	r16, r0
	sts	pprint+3, r16
	lds	r16, pprint+4
	lds	r0, pextended+2
	adc	r16, r0
	sts	pprint+4, r16
	lds	r16, pprint+5
	lds	r0, pextended+3
	adc	r16, r0
	sts	pprint+5, r16

fdiskprintentry130:

	ldd	r16, Z+M_PartSize+0
	sts	pprint+6, r16
	ldd	r16, Z+M_PartSize+1
	sts	pprint+7, r16
	ldd	r16, Z+M_PartSize+2
	sts	pprint+8, r16
	ldd	r16, Z+M_PartSize+3
	sts	pprint+9, r16
	
	call	print
	.db	0x90,": ", 0x81," 1023 254  63 - 1023 254  63 [0x", 0x85, 0x84, 0x83, 0x82, " - 0x", 0x89, 0x88, 0x87, 0x86, "] ", 0x00
	ldd	r16, Z+M_PartType
	cpi	r16, 0x00
	brne	fdiskprintentry010
	call	print
	.db	"unused ", CR, LF, 0x00
	ret
fdiskprintentry010:
	cpi	r16, 0x83
	brne	fdiskprintentry020
	call	print
	.db	"Linux files", CR, LF, 0x00
	ret

fdiskprintentry020:
	cpi	r16, 0x05
	brne	fdiskprintentry030
	call	print
	.db	"Extended DOS ", CR, LF, 0x00
	ret

fdiskprintentry030:
	cpi	r16, 0x0b
	brne	fdiskprintentry040
	call	print
	.db	"Win95 FAT-32 ", CR, LF, 0x00
	ret

fdiskprintentry040:
	cpi	r16, 0x06
	brne	fdiskprintentry050
	call	print
	.db	"Win95 FAT-16b", CR, LF, 0x00
	ret

fdiskprintentry050:
	cpi	r16, 0x07
	brne	fdiskprintentry060
	call	print
	.db	"extended FAT ", CR, LF, 0x00
	ret

fdiskprintentry060:
	cpi	r16, 0x01
	brne	fdiskprintentry070
	call	print
	.db	"MS-DOS FAT12 ", CR, LF, 0x00
	ret
fdiskprintentry070:
	call	print
	.db	"other", CR, LF, 0x00
	ret
