;--------------------------------------------------------------------------
;
;	User load and store routines for AVR Monitor
;
;--------------------------------------------------------------------------
.macro	loadusr
.endmacro

.macro	storeusr
.endmacro

;--------------------------------------------------------------------------
;
;	AVR Monitor, inlcude the default commands followed by application
;	specific commands
;
#define clibuffer InputBuffer		; use this buffer for monitor
#define charout   serout
#define charin    serin
.include	"../include/monitor-chartbl-v1-3.asm"
	.db	0, 'I'			; CMD0
	.db	0, 'H'			; CMD8
	.db	0, 'J'			; ACMD41
	.db	0, 'K'			; CMD58
	.db	0, 'L'			; CMD16 
	.db	0, 'R' 			; Read Sector
	.db	0, 'W' 			; Write Sector
;
	.db	0, 'Q'			; Volume Info
	.db	0, 'T'			; Linked Cluster
	.db	0, 'Y'			; Read Block
	.db	0, 'R' & 0x1F		; DMA read ^R is now redraw !!
	.db	0, 'W' & 0x1F		; DMA write
	.db	0, 'U'       		; Memcheck
	.db	0, 'P' & 0x1F		; patternw
	.db	0, 'D' & 0x1F		; Update DD
	.db	0, 'X' & 0x1F		; Radix-50
;
;	AVR Monitor, inlcude the action routine address table of the default
;	commands followed by the addresses of the application specific 
;	command routines. Both tables must be in sync of course...
;
.include	"../include/monitor-subtbl-v1-3.asm"
	.dw	mon_sd_card_spi
	.dw	mon_sd_card_ifc
	.dw	mon_sd_card_init
	.dw	mon_sd_card_readocr
	.dw	mon_sd_card_blklen
	.dw	mon_card_read
	.dw	mon_card_write
;
	.dw	volume
	.dw	linkedclust
	.dw	readblock
	.dw	dmareadpdp11
	.dw	dmawritepdp11
	.dw	memcheck
	.dw	patternw
	.dw	updatedd
	.dw	radix50

;
;	AVR monitor. A interactive monitor inspired by the Apple II
;	this is a special version just for the RLV12 Emulator V2.0
;
.include "monitor-v1-6a.asm"
;
;	Store incrementing values to the range given starting with 0 this
;	is used to create a pattern in RAM to check that the ROM reading
;	logic is correct.
;
;	Set SD-card into SPI mode
;
mon_sd_card_spi:			; Monitor wrapper
	ldi	yl, low(pcluster)	; Setup parameter block for general IO
	ldi	yh, high(pcluster)	; 
	call	SD_CARD_SPI
	brcs	mon_sd_card_spi_err
	ldd	r16, Y+P_IOStatus+0
	sts	pprint+0, r16
	ldd	r16, Y+P_Duration+0
	sts	pprint+12, r16
	call	print
	.db	"Set SPI Mode:", 0x8a, " Counts:", 0x8c, 0x0d, 0x0a, 0x00
	clc
	ret
mon_sd_card_spi_err:
	call	print
	.db	"Set SPI Mode failed!", 0x07, 0x0d, 0x0a, 0x00
	sec
	ret
;
;	Read Interface Condition
;
mon_sd_card_ifc:			; Monitor wrapper
	ldi	yl, low(pcluster)	; Setup parameter block for general IO
	ldi	yh, high(pcluster)	; 
	call	SD_CARD_IFC
	brcc	mon_sd_card_ifcv2
	ldd	r16, Y+P_IOStatus+0
	sts	pprint+0, r16
	call	print
	.db	"Interface Condition Error:", 0x8a, 0x0a, 0x0d, 0x00
	sec
	ret

mon_sd_card_ifcv2:
	rcall	mon_sd_card_copy
	call	print
	.db	"Interface Condition Read:0x", 0x80, 0x81, 0x82, 0x83, 0x84, 0x0a, 0x0d, 0x00, 0x00
	clc
	ret

mon_sd_card_copy:
	lds	r16, sd_rsp
	sts	pprint, r16
	lds	r16, sd_rsp+1
	sts	pprint+1, r16
	lds	r16, sd_rsp+2
	sts	pprint+2, r16
	lds	r16, sd_rsp+3
	sts	pprint+3, r16
	lds	r16, sd_rsp+4
	sts	pprint+4, r16
	lds	r16, sd_rsp+5
	sts	pprint+5, r16
	ret

;
;	Read Operational Condition Register
;
mon_sd_card_readocr:
	ldi	yl, low(pcluster)	; Setup parameter block for general IO
	ldi	yh, high(pcluster)	; 
	call	SD_CARD_READOCR
	rcall	mon_sd_card_copy
	lds	r16, sd_status
	sts	pprint+6, r16
	call	print
	.db	"Operation Condition Reg.:0x", 0x80, 0x81, 0x82, 0x83, 0x84, 0x85, " SD_Status:0x", 0x86, 0x0a, 0x0d, 0x00
	ret
;
;	Initialise Card
;
mon_sd_card_init:
	ldi	yl, low(pcluster)	; Setup parameter block for general IO
	ldi	yh, high(pcluster)	; 
	call	SD_CARD_INIT

	ldd	r16, Y+P_IOStatus+0
	sts	pprint+0, r16
	ldd	r16, Y+P_IOStatus+1
	sts	pprint+1, r16

	brcs	mon_sd_card_init_e
	call	print
	.db	"Initialise Card RC:0x", 0x8a, ", # of tries:0x", 0x81, 0x0a, 0x0d, 0x00, 0x00
	clc
	ret
mon_sd_card_init_e:
	call	print
	.db	"Initialise Card Error:", 0x80, 0x0a, 0x0d, 0x00
	sec
	ret
;
;	Set Block Length
;
mon_sd_card_blklen:
	ldi	yl, low(pcluster)	; Setup parameter block for general IO
	ldi	yh, high(pcluster)	; 
	call	SD_CARD_BLKLEN
	call	print
	.db	"Set Block Length RC:", 0x8a, 0x0a, 0x0d, 0x00
	ret
;
;	xxxx<yyyy^R	Read Sector yyyy to buffer at xxxx
;	xxxx<yyyyX	Read Block yyyy to buffer at xxxx using fraglist
;
;	a4	= Buffer Address
;	a1	= Sector Address
;
	lds	r16, a4l
	std	Y+P_Address, r16
	lds	r16, a4h
	std	Y+P_Address+1, r16
	ret

readblock:
	ldi	yl, low(pcluster)
	ldi	yh, high(pcluster)
	lds	r16, a1l
	std	Y+P_Cluster, r16
	lds	r16, a1h
	std	Y+P_Cluster+1, r16
	lds	r16, a1b
	std	Y+P_Cluster+2, r16
	std	Y+P_Cluster+3, zero
	call	Logical2Physical
	brcc	mon_card_sector
;
;
;


mon_card_read:
	ldi	yl, low(pcluster)
	ldi	yh, high(pcluster)
	lds	r16, a1l
	std	Y+P_Sector+0, r16
	lds	r16, a1h
	std	Y+P_Sector+1, r16
	lds	r16, a1b
	std	Y+P_Sector+2, r16
	std	Y+P_Sector+3, zero
;
mon_card_sector:
	lds	r16, a4l
	std	Y+P_Address, r16
	lds	r16, a4h
	std	Y+P_Address+1, r16

	std	Y+P_IOStatus+0, zero
	std	Y+P_IOStatus+1, zero
	std	Y+P_Duration+0, zero
	std	Y+P_Duration+1, zero

	call	SD_CARD_READ
	brcc	mon_card_read_ok

	ldd	r16, Y+P_IOStatus+0
	sts	pprint+0, r16
	call	print
	.db	"Error on Read:0x", 0x80, 0x0a, 0x0d, 0x00
	rjmp	mon_card_read_exit

mon_card_read_ok:
	ldd	r16, Y+P_IOStatus+0	; Copy over read CRC
	ldd	r17, Y+P_IOStatus+1
	sts	pprint+0, r16
	sts	pprint+1, r17
	ldd	r16, Y+P_IOStatus+2	; Copy over calculated CRC
	ldd	r17, Y+P_IOStatus+3
	sts	pprint+2, r16
	sts	pprint+3, r17
	lds	r16, a4l		; Advance Buffer Addresss
	lds	r17, a4h
	subi	r16, low(-512)
	sbci	r17, high(-512)
	sts	a4l, r16
	sts	a4h, r17
	
	
	push	r4
	push	r5

	ldi	r16, 23			; 1 tick equals 11.5usec so we first
	ldd	r0, Y+P_Duration+0	; multiply by 23 and then devide by 2.
	mul	r0, r16
	movw	r5:r4, r1:r0
	ldd	r0, Y+P_Duration+1
	mul	r0, r16
	add	r5, r0
	adc	r1, zero
	
	lsr	r1
	ror	r5
	ror	r4
	sts	pprint+4, r4
	sts	pprint+5, r5
	
	pop	r5
	pop	r4
	call	print
	.db	"Read CRC:0x", 0x81, 0x80, " Calc:0x", 0x82, 0x83, " Time.:", 0xc4, "usec", CR, LF, 0
mon_card_read_exit:
	ret
	
;
;	xxxx<yyyy^W	Write Buffer at yyyy to Sector xxxx
;
;	a4	= Sector Address
;	a1	= Buffer Address
;
mon_init_write:
	ldi	yl, low(pcluster)
	ldi	yh, high(pcluster)
	lds	r16, a4l
	std	Y+P_Sector+0, r16
	lds	r16, a4h
	std	Y+P_Sector+1, r16
	lds	r16, a4b
	std	Y+P_Sector+2, r16
	std	Y+P_Sector+3, zero
	lds	r16, a1l
	std	Y+P_Address, r16
	lds	r16, a1h
	std	Y+P_Address+1, r16
	ret

mon_card_write:
	rcall	mon_init_write
	call	SD_CARD_WRITE
	ldd	r16, Y+P_IOStatus+1
	sts	pprint+1, r16
	brcs	mon_card_write_err
	
	lds	r16, a1l
	subi	r16, low(-512)
	sts	a1l, r16
	lds	r16, a1h
	sbci	r16, high(-512)
	sts	a1h, r16
	
	call	print
	.db	"Write Status:0x", 0x8a, ", 0x", 0x8b, " Time(in ~11usec):0x", 0x8d, 0x8c, 0x0a, 0x0d, 0x00
	ret

mon_card_write_err:
	call	print
	.db	"Write Error:0x", 0x8a, ", 0x", 0x8b, 0x0a, 0x0d, 0x00, 0x00
	ret

;
;	Mount all Volumes and Display FAT Volume Information
;
volume:
	call	MountVolume
	sts	pprint+12, r24
	brcc	volumeok
	call	print
	.db	"IO Error:", 0x8a, ", ", 0x8b, " Error ID:", 0x8c, 0x0a, 0x0d, 0x00, 0x00
	clc
	ret

volumeok:
	lds	r0, VolStatus
	sts	pprint, r0
	call	print
;		 ----+----1----+----2----+----3----+----4
	.db	"Status................................:", 0x80, 0x0a, 0x0d, 0x00, 0x00

	lds	r0, part1start+3
	sts	pprint+3, r0
	lds	r0, part1start+2
	sts	pprint+2, r0
	lds	r0, part1start+1
	sts	pprint+1, r0
	lds	r0, part1start+0
	sts	pprint+0, r0
	call	print
	.db	"Partition start.......................:", 0x83, 0x82, 0x81, 0x80, 0x0a, 0x0d, 0x00	


	lds	r0, sectbefore+3
	sts	pprint+3, r0
	lds	r0, sectbefore+2
	sts	pprint+2, r0
	lds	r0, sectbefore+1
	sts	pprint+1, r0
	lds	r0, sectbefore+0
	sts	pprint+0, r0
	call	print
	.db	"Sectors before this partition.........:", 0x83, 0x82, 0x81, 0x80, 0x0a, 0x0d, 0x00	

	lds	r0, sectpart+3
	sts	pprint+3, r0
	lds	r0, sectpart+2
	sts	pprint+2, r0
	lds	r0, sectpart+1
	sts	pprint+1, r0
	lds	r0, sectpart+0
	sts	pprint+0, r0
	.db	"Partition size........................:", 0x83, 0x82, 0x81, 0x80, 0x0a, 0x0d, 0x00	

	lds	r0, numfat
	sts	pprint, r0
	call	print
	.db	"Number of FATs........................:", 0x80, 0x0a, 0x0d, 0x00, 0x00

	lds	r0, sectperfat+3
	sts	pprint+3, r0
	lds	r0, sectperfat+2
	sts	pprint+2, r0
	lds	r0, sectperfat+1
	sts	pprint+1, r0
	lds	r0, sectperfat+0
	sts	pprint+0, r0
	call	print
	.db	"Sectors per FAT.......................:", 0x83, 0x82, 0x81, 0x80, 0x0a, 0x0d, 0x00	

	lds	r0, sectperclst
	sts	pprint+0, r0
	call	print
	.db	"Sectors per cluster...................:", 0x80, 0x0a, 0x0d, 0x00, 0x00

	lds	r0, bytespsect+1
	sts	pprint+1, r0
	lds	r0, bytespsect
	sts	pprint+0, r0
	call	print
	.db	"Bytes per sector......................:", 0x81, 0x80, 0x0a, 0x0d, 0x00	

	lds	r0, reservdsect+1
	sts	pprint+1, r0
	lds	r0, reservdsect
	sts	pprint+0, r0
	call	print
	.db 	"Reserved sectors......................:", 0x81, 0x80, 0x0a, 0x0d, 0x00	

	lds	r0, fat1start+3
	sts	pprint+3, r0
	lds	r0, fat1start+2
	sts	pprint+2, r0
	lds	r0, fat1start+1
	sts	pprint+1, r0
	lds	r0, fat1start+0
	sts	pprint+0, r0
	call	print
	.db	"1st FAT starts at sector..............:", 0x83, 0x82, 0x81, 0x80, 0x0a, 0x0d, 0x00	

	lds	r0, datastart+3
	sts	pprint+3, r0
	lds	r0, datastart+2
	sts	pprint+2, r0
	lds	r0, datastart+1
	sts	pprint+1, r0
	lds	r0, datastart+0
	sts	pprint+0, r0
	call	print
	.db	"Data starts at sector.................:", 0x83, 0x82, 0x81, 0x80, 0x0a, 0x0d, 0x00	

	lds	r0, VolStatus
	sbrc	r0, FAT32
	rjmp	volume32info

	lds	r0, dirsectors
	sts	pprint+0, r0
	call	print
	.db	"FAT16 Volume with root dir sectors....:", 0x80, 0x0a, 0x0d, 0x00, 0x00
	
	lds	r0, rootdir+3
	sts	pprint+3, r0
	lds	r0, rootdir+2
	sts	pprint+2, r0
	lds	r0, rootdir+1
	sts	pprint+1, r0
	lds	r0, rootdir+0
	sts	pprint+0, r0
	call	print
	.db	"The root dir starts at sector.........:", 0x83, 0x82, 0x81, 0x80, 0x0a, 0x0d, 0x00
	ret

volume32info:
	lds	r0, rootdir+3
	sts	pprint+3, r0
	lds	r0, rootdir+2
	sts	pprint+2, r0
	lds	r0, rootdir+1
	sts	pprint+1, r0
	lds	r0, rootdir+0
	sts	pprint+0, r0
	call	print
	.db	"FAT32 Volume root dir start cluster...:", 0x83, 0x82, 0x81, 0x80, 0x0a, 0x0d, 0x00
	ret

;==========================================================================
;
;
;
;--------------------------------------------------------------------------
;
;		"avrmem"<"pdp11memstart"."pdp11memend"R
;
dmareadpdp11:
;
;	DMA Address is registers 4, 5, 7
;
	lds	r16, a1l
	lds	r17, a1h
	lds	r18, a1b
	ori	r16, 0x01
	dmaaddr	r16, r17, r18
dmareadrqst:
	dmaread	r18, r19		; Monitor routines use r16 and r17 !!
	brcs	dmareadtmo
;
;	now we have the Q-Bus data in r18 and r19
;
	lds	xl, a4l
	lds	xh, a4h
	mov	r17, r18
	call	store			; Store the word in MCU memory
	call	nxta4
	mov	r17, r19
	lds	xl, a4l
	lds	xh, a4h
	call	store
	call	nxta4
	brlo	dmareadrqst		; More to go
	ret				; done

dmareadtmo:
	rjmp	dmaprinta1
;--------------------------------------------------------------------------
;
;		"pdp11mem"<"avrmemstart"."avrmemend"W
;
dmawritepdp11:
	lds	r16, a4l
	lds	r17, a4h
	lds	r18, a4b
	andi	r16, 0xFE
	dmaaddr	r16, r17, r18
	
dmawriterqst:
	lds	xl, a1l
	lds	xh, a1h
	call	load
	push	r17			; Save Lower Byte
	adiw	xl:xh, 1
	call	load			; Higher byte is in R17
	pop	r16			; Restore Lower Byte
	dmawrt	r16, r17
	brcs	dmawritetmo
	call	nxta4
	call	nxta4
	brlo	dmawriterqst
	ret
	
dmawritetmo:
	call	pra1
	rjmp	dmaprinta1

;--------------------------------------------------------------------------
;
;		xxxU	check 64kword of RAM, fill with value 0..0xffff and check
;
memwritetmo:
	call	pra1
	rjmp	dmaprinta1

memcheck:
	clr	r16			; DMA Write
	clr	r17
	lds	r18, a1l
	dmaaddr	r16, r17, r18
	ldi	r19, 64		; We report only 128 errors then we stop
	clr	xl
	clr	xh

memwriterqst:

	dmawrt	xl, xh
	brcs	memwritetmo
	adiw	xh:xl, 1
	brne	memwriterqst
;
	ldi	r16, 0x01
	clr	r17
	lds	r18, a1l
	dmaaddr	r16, r17, r18
memcheckrqst:
	dmaread	r16, r17
	brcs	memchecktmo
	cp	xl, r16
	cpc	xh, r17
	breq	memchecknext
	rcall	memcheckfailed
	inc	r19
	brmi	memcheckdone
memchecknext:
	adiw	xh:xl, 1
	brne	memcheckrqst
	cpi	r19, 64
	brne	memcheckdone
	call	print
	.db	"Memory checked successfully", 0x0a, 0x0d, 0x00
memcheckdone:
	ret

memchecktmo:
	call	pra1
	rjmp	dmaprinta1

memcheckfailed:
	sts	pprint+0, xl
	sts	pprint+1, xh
	sts	pprint+2, r16
	sts	pprint+3, r17

	mov	r16, xl
	lsl	r16
	sts	pprint+4, r16
	mov	r16, xh
	rol	r16
	sts	pprint+5, r16
	lds	r16, a1l
	adc	r16, zero
	sts	pprint+6, r16

	call	print
	.db	"Memory check failed at:", 0xB4, " read:", 0xA2, " expectd:", 0xA0, 0x0a, 0x0d, 0x00
	ret


dmaprinta1:
	lds	r16, a1l
	lds	r17, a1h
	sts	pprint+0, r16
	sts	pprint+1, r17
	call	print
	.db	CR, LF, 0x81, 0x80, "?", 0x07, 0, 0
	ret
;--------------------------------------------------------------------------
;
;	Update DD register (aka CSR, CS1, etc.)
;
updatedd:
	lds	r16, a1l
	lds	r17, a1h
	sts	CSRL, r16
	sts	CSRH, r17
	ret
;--------------------------------------------------------------------------
;
;	Word Pattern
;
;	Store incrementing values to the range given starting with 0 this
;	is used to create a pattern in RAM to check that the ROM reading
;	logic is correct.
;
patternw:
	push	r24
	push	r25
	clr	r24
	clr	r25
patternw010:
	lds	xl, a1l
	lds	xh, a1h
	st	X+, r24
	st	X+, r25
	adiw	r24:r25, 1
	call	nxta1
	call	nxta1
	brlo	patternw010
	pop	r25
	pop	r24
	ret
;--------------------------------------------------------------------------
;
;	Decode a 16 bit RADIX50 to ASCII
;
radix50cvt:
	.db	" ABCDEFGHIJKLMNOPQRSTUVWXYZ$.%0123456789"
radix50:
	
	push	xl
	push	xh
	push	yl
	push	yh
	push	zl
	push	zh
	
	lds	xl, a1l
	lds	xh, a1h

	ldi	yl, low(050*050)
	ldi	yh, high(050*050)
	
	clr	r16
radix010:
	cp	xl, yl
	cpc	xh, yh
	brlo	radix020
	sub	xl, yl
	sbc	xh, yh
	inc	r16
	rjmp	radix010

radix020:
	mov	zl, r16
	clr	zh
	subi	zl, low(-2*radix50cvt)
	sbci	zh, high(-2*radix50cvt)
	lpm	r16, Z
	sts	pprint+0, r16

	ldi	yl, low(050)
	ldi	yh, high(050)
	clr	r16
radix030:
	cp	xl, yl
	cpc	xh, yh
	brlo	radix040
	sub	xl, yl
	sbc	xh, yh
	inc	r16
	rjmp	radix030

radix040:
	mov	zl, r16
	clr	zh
	subi	zl, low(-2*radix50cvt)
	sbci	zh, high(-2*radix50cvt)
	lpm	r16, Z
	sts	pprint+1, r16

	mov	zl, xl
	clr	zh
	subi	zl, low(-2*radix50cvt)
	sbci	zh, high(-2*radix50cvt)
	lpm	r16, Z
	sts	pprint+2, r16

	call	print
	.db	0x90, 0x91, 0x92, 0x0d, 0x0a, 0x00

	pop	zh
	pop	zl
	pop	yh
	pop	yl
	pop	xh
	pop	xl
	ret


;==========================================================================
;
linkedclust:
	ldi	yl, low(pcluster)
	ldi	yh, high(pcluster)
	
	lds	r16, a1l
	std	Y+P_Cluster+0, r16
	lds	r16, a1h
	std	Y+P_Cluster+1, r16
	lds	r16, a1b
	std	Y+P_Cluster+2, r16
	std	Y+P_Cluster+3, zero

	call	LinkedCluster

	lds	r16, a1l
	sts	pprint+0, r16
	lds	r16, a1h
	sts	pprint+1, r16
	lds	r16, a1b
	sts	pprint+2, r16
	sts	pprint+3, zero

	ldd	r16, Y+P_Cluster+0
	sts	pprint+4, r16
	ldd	r16, Y+P_Cluster+1
	sts	pprint+5, r16
	ldd	r16, Y+P_Cluster+2
	sts	pprint+6, r16
	ldd	r16, Y+P_Cluster+3
	sts	pprint+7, r16
	call	print
	.db	"Cluster 0x", 0x83, 0x82, 0x81, 0x80, " links to cluster 0x", 0x87, 0x86, 0x85, 0x84, 0x0d, 0x0a, 0x00, 0x00
	ret
