;--------------------------------------------------------------------------
;
;	Read INIT file
;
readinitname:
	.db		"RLV12.INI", 0
;
readshow:
	push	xl
	push	xh
	call	print
	.db	"Init: Executing Command:'", 0
	movw	xh:xl, r5:r4
readshow010:
	ld	r24, X+
	cpi	r24, CR
	breq	readshow020
	call	serout
	rjmp	readshow010
readshow020:
	call	print
	.db	"'", CR, LF, 0
	pop	xh
	pop	xl
	ret

;--------------------------------------------------------------------------
;
;	Note: readinit is part of the sdcardinsert process and hence
;	it must not change any register except r24, r25, because
;	sdcardinsert is called by the scheduler which is part of the
;	serin function and all callers of serin expect that nothing
;	except r24, r25 is changed.
;
;	readinit is called by MountVolume and the cli using the
;	'readinit' command. MountVolume already saves the following
;	registers: r16, r17, r18, r19, r20, xl, xh, yl, yh, zl, zh
;	therefore we don't have to save them again. The cli interface
;	expects only xl, xh to be preserved
;
readinit:
	push	r4			; We need some preserved registers
	push	r5
	push	xl			; We must preserve xh:xl because of CLI
	push	xh
	sts	rlv12init, zero
	ldi	r24, low(256)
	ldi	r25, high(256)
	call	malloc
	sbiw	r25:r24, 0
	brne	readinit005
	call	print
	.db	"Init: Failed to allocated Buffer.", CR, LF, 0
	ldi	r24, ERR_MALLOC	
	rjmp	readinitfail

readinit005:
	movw	r5:r4, r25:r24
	ldi	zl, low(2*readinitname)
	ldi	zh, high(2*readinitname)
	movw	xh:xl, r5:r4

readinit010:
	lpm	r16, Z+
	st	X+, r16
	tst	r16
	brne	readinit010
	ldi	yl, low(pdircluster)
	ldi	yh, high(pdircluster)
	movw	xh:xl, r5:r4
	call	startclustersave
	call	Name2DirEntry
	brcc	readinit030
	call	print
	.db	"Init: RLV12.INI not found", CR, LF, 0
	call	startclusterrest
	ldi	r24, ERR_FNF
	rjmp	readinitexit
	
readinit030:
	ldd	zl, Y+P_DirPointer+0
	ldd	zh, Y+P_DirPointer+1
	ldd	r16, Z+D_Attr
	sbrs	r16, A_Directory
	rjmp	readinit040
	call	print
	.db	"Init: RLV12.INI not a valid file", CR, LF, 0, 0
	call	startclusterrest
	ldi	r24, ERR_NAF
	rjmp	readinitexit

readinit040:
	call	ReadFileOpen
	brcc	readinit050
	sts	pprint+0, r24
	call	print
	.db	"Error 0x", 0x80, " opening file RLV12.INI ", CR, LF, 0
	ldi	r24, ERR_OPEN
	rjmp	readinitexit

readinit050:
	call	print
	.db	"Init: Found RLV12.INI", CR, LF, 0

readinit060:
	movw	xh:xl, r5:r4
	clr	r18			; No bytes in command
readinit070:
	call	ReadFileByte
	brcs	readinit090		; Most likely end-of-file
	cpi	r24, LF
	breq	readinit080
	cpi	r24, CR
	breq	readinit080
	sbrc	r24, 7
	rjmp	readinit060		; Invalid Character, ignore line
	st	X+, r24
	inc	r18
	brpl	readinit070

	call	print
	.db	"Init: Fatal overflow!", CR, LF, 0
	ldi	r24, ERR_TOLONG
	rjmp	readinitexit

readinit080:
	tst	r18			; Anything in command line
	breq	readinit060		; Not really
	ldi	r16, CR
	st	X, r16
	rcall	readshow

	push	yl			; Save fcb
	push	yh
	movw	xh:xl, r5:r4
	ldi	zl, low(2*commandlist)	; Parser table
	ldi	zh, high(2*commandlist)
	sts	tpflags, zero
	call	scancommand
	pop	yh
	pop	yl

	rjmp	readinit060
	
readinit090:
	cpi	r24, 0x1A		; ^Z?
	breq	readinit100		; No it's another error
	sts	pprint+0, r24
	call	print
	.db	"Init: Error 0x", 0x80, " reading RLV12.INI", CR, LF, 0
	ldi	r24, ERR_READ
	rjmp	readinitexit
	
readinit100:
	tst	r18
	breq	readinit110		; Nothing to do
	ldi	r16, CR
	st	X, r16
	rcall	readshow
	
	push	yl			; Save fcb
	push	yh
	movw	xh:xl, r5:r4
	ldi	zl, low(2*commandlist)	; Parser table
	ldi	zh, high(2*commandlist)
	sts	tpflags, zero
	pop	yh
	pop	yl

	call	scancommand
readinit110:
	call	ReadFileClose
	ldi	r24, IO_SUCCESS
readinitexit:
	push	r24
	movw	r25:r24, r5:r4
	call	free
	pop	r24
readinitfail:
	pop	xh
	pop	xl
	pop	r5
	pop	r4
	ret
