;
;	We should no longer make use of registers reserved for the ISR
;	instead we should use the standard interrupt frame of the RTOS
;	and at the same time no longer assume the regisers ff, zero and
;	one have valid contents during interrupt. This will add another
;	20-24 cycles. This should still work
;
;--------------------------------------------------------------------------
;
;	Q-Bus Interface Interrupt Service Routines 
;
;	Device Register Access
;	Boot ROM Access
;	Interrupt Acknowledge
;
;--------------------------------------------------------------------------
;
;	General Macro's
;
.macro	DATI
;
;	Write to Q-Bus Data Registers, original version
;
	cbi	b_RD			;  2 Finish pending read
	out	dataportdir, ff		;  1 set port direction to output
	out	dataportout, zero	;  1 Q-Bus Data Registers
	sbi	b_ALEW			;  2
	cbi	b_ALEW			;  2
	out	dataportout, @0		;  1 write low byte
	sbi	b_WR			;  2
	cbi	b_WR			;  2
	out	dataportout, @1		;  1 write high byte
	sbi	b_WR			;  2
	cbi	b_WR			;  2
					;---
					; 18 cycles
.endmacro
;
;	Interrupt Cleanup as a macro, we replicate the code to avoid calls and 
;	branches to save CPU cycles
;
.macro	DATI_exit
	sbis	GPIOR0, log__reg	;  2 Logging requires more cycles
	rjmp	nologging		;
	lds	zl, log_pointer+0	;  3 Logging
	lds	zh, log_pointer+1	;  3 Logging
	std	Z+0, one		;  1 1	= Valid Log Entry with FNC=Maint
	std	Z+1, log_csrread	;  1 0	= Read
	ldi	itemp, @2/2		;  1
	std	Z+2, @0			;  1 low byte
	std	Z+3, @1			;  1 high byte
	std	Z+4, itemp		;  1 address
	lds	itemp, TCNT2		;  3 Get ticks
	std	Z+5, itemp		;  1 Store ticks
	lognxt	zl, zh			;  8 next address address
nologging:
	sbi	b_IACK			;  2
	cbi	b_IACK			;  2
	pop	zh			;  2
	pop	zl			;  2
	out	SREG, isreg		;  1
	.if	signal==1
	cbi	b_SIG
	nop
	sbi	b_SIG
	nop
	cbi	b_SIG
	.endif
	reti				;  5
					;---
					; 40
.endmacro
;
;
;
.macro	DATO
;
;	Read from Q-Bus, original version
;
	sbi	b_ALER			;  2
	cbi	b_ALER			;  2
	nop				;  1
	nop				;  1
	nop				;  1
	in	@0, dataportin		;  1
	sbi	b_ALER			;  2
	cbi	b_ALER			;  2
	nop				;  1
	nop				;  1
	nop				;  1
	in	@1, dataportin		;  1
					;---
					; 16
.endmacro

.macro	DATO_exit			;;;
;
;	Interrupt Cleanup as a macro, we replicate the code to avoid calls and 
;	branches to save CPU cycles
;
	sbis	GPIOR0, log__reg	;  2
	rjmp	nologging
	lds	zl, log_pointer+0	;  3 Logging
	lds	zh, log_pointer+1	;  3 Logging
	std	Z+0, one		;  2 1	= Valid Log Entry with FNC=Maint
	std	Z+1, log_csrwrite	;  2 1	= Write Device Register
	std	Z+2, @0			;  2 low byte
	std	Z+3, @1			;  2 high byte
	ldi	itemp, @2/2		;  1
	std	Z+4, itemp		;  2 address
	lds	itemp, TCNT2		;  3 Get ticks
	std	Z+5, itemp		;  2 Store ticks
	lognxt	zl, zh			;  8 next address address
nologging:
	sbi	b_IACK			;  2 Signal the CPLD to assert RPLY
	cbi	b_IACK			;  2
	pop	zh			;  2 Restore Z pointer
	pop	zl			;  2
	out	SREG, isreg		;  1 Restore processor status
	.if	signal==1
	cbi	b_SIG
	nop
	sbi	b_SIG
	nop
	cbi	b_SIG
	.endif
	reti				;  5 Exit interrupt
					;---
					; 46
.endmacro
;--------------------------------------------------------------------------
;
;	Device Register Access Routines
;
;	Interrupt input INT0 is used for DATI/DATO Q-Bus cycles. Whenever
;	the PDP-11 reads or writes one of the implemented registers in
;	the IO page the CPLD de-asserts INT0. INT0 is configured as low
;	input level interupt.
;	First the ISR reads register 0 which has the following content
;	
;	+---+---+---+---+---+---+---+---+
;	| UB| LB|ROM|BA4|BA3|BA2|BA1| WT|
;	+---+---+---+---+---+---+---+---+
;
;	WT	this indicates whether the Q-Bus cycle is a DATI (WT=0)
;		or a DATO/DATOB (WT=1) cycle.
;	BA1..4	these are the bus address bits A1..4
;	ROM	if set indicates that the ROM regiona 173xxx was accessed
;	LB	if cleared means the lower byte is written by DATO/DATOB
;	UB	if cleared means the upper byte is written by DATO/DATOB
;		in case both, LB and UB, are cleared this is a DATO cycle
;
.equ	UB	= 7			; Don't Write Upper Byte
.equ	LB	= 6			; Don't Write Lower Byte
.equ	ROM	= 5			; Boot ROM
.equ	WTBT	= 0			; Write i.e. DATO
;
INTQ_isr:				;  7 
	.if	signal==1
	sbi	b_SIG
	.endif
;
	in	isreg, SREG		;  1 Save processor status
;
	.if debug==1
	lds	itemp, INTQ_count	;    Q-Bus Interrupt Count
	inc	itemp			;   
	sts	INTQ_count, itemp	;    
	.endif
;
	cbi	b_RD			;  2 Make Sure we are in non-read mode
	out	dataportdir, ff		;  1 Set Data Port Direction
	out	dataportout, zero	;  1 Set Register Select Start
	sbi	b_ALER			;  2 Latch Register Select
	cbi	b_ALER			;  2
	out	dataportdir, zero	;  1 Switch Data Port to Input
	sbi	b_RD			;  2 Enable CPLD Outputs
	push	zl			;  2 Use the time to wait for the input
	push	zh			;  2 to synchronise to save registers now
	in	zl, dataportin		;  1 retrieve device register address
;
;	Now we check if this is an access to the ROM section in the IO-page
;
	sbrc	zl, ROM			;  2 ROM/CSR select
	rjmp	INTQ_ROM		;    this is for the Boot ROM
;
;	if the controller is busy we do not process Q-Bus cycles
;
	sbis	b_CRDY			;  2 skip if controller ready
	rjmp	INTQ_busy
;
;	use WT and BA1..3 as index into jump table to displatch register
;
	andi	zl, 0x0F		;  1 Mask out A3, A2, A1, WTBT
	clr	zh			;  1
	subi	zl, low(-INTQ_table)	;  1
	sbci	zh, high(-INTQ_table)	;  1 and convert to index into jump table
	ijmp				;  2 and execute device registers IO
;
INTQ_table:
	rjmp	csrin			;  2 WTBT is low for DATI and high for DATO
	rjmp	csrout
	rjmp	barin
	rjmp	barout
	rjmp	darin
	rjmp	darout
	rjmp	mprin
	rjmp	mprout
	rjmp	baein
	rjmp	baeout
	rjmp	bae2in
	rjmp	bae2out
	rjmp	bae4in
	rjmp	bae4out
	rjmp	bae6in
	rjmp	bae6out
					;---
					; 36 cycles
;
;	Interrupt Exit in case the Controller is busy
;
INTQ_busy:
	.if debug==1
	lds	itemp, INTB_count	;;; Controller Busy Interrupt count
	inc	itemp
	sts	INTB_count, itemp
	.endif
	pop	zh
	pop	zl
	out	SREG, isreg
;
	.if	signal==1
	cbi	b_SIG
	nop
	sbi	b_SIG
	nop
	cbi	b_SIG
	nop
	sbi	b_SIG
	nop
	cbi	b_SIG
	.endif
;
	reti

;--------------------------------------------------------------------------
;
;	CSR		17774400
;
;	+---+---+---+---+---+---+---+---++---+---+---+---+---+---+---+---+
;	|ERR|DE |E3 |E2 |E1 |E0 |DS1|DS0||CRY|IE |A17|A16|F2 |F1 |F0 |RDY|
;	+---+---+---+---+---+---+---+---++---+---+---+---+---+---+---+---+
;
;	Bit(s) 	Description
;	0	Drive Ready (DRDY) - When set, this bit indicates that the selected  
;		drive is ready to receive a command. The bit is cleared when a seek
;		or head-select operation is initiated and set when the operation is
;		completed.
;
;	1-3 	Function Code - These bits are set by software to indicate the  
;		command to be executed.
;		F2-F0	Command
;		000	NOP (RL11) Maintenance Mode (RLV11/RLV12)
;		001	Write Check
;		010	Get Status
;		011	Seek
;		100	Read Header
;		101	Write Data
;		110	Read Data
;		111	Read Data Without Header Check
;
;	4-5	Bus Address Extension Bits (BA16, BA17) - The two most significant bus
;		address bits when operating in 18-bit addressing modes. Re,ad and      
;		written as data bits 4 and 5 of theCS register but considered as      
;		address bits 16 and 17 of the bus address register (see Paragraph     
;		3.4.2).                                                               
;
;	6	Interrupt Enable (IE) - When this bit is set by software, the controller 
;		is allowed to interruptthe processor at the normal command or error 
;		termination ..
;
;	7	Controller Ready (CRDY) - When cleared by software, this bit indicates 
;		that the commandcode in bits 1-3 is to be executed (negative GO bit). 
;		When set, this bit indicates the controller is ready to accept another 
;		command.
;
;	8-9	Drive Select (DS0, DS1) - These bits determine which drive will 
;		communicate with the controller via the drive bus.
;
;	10-13	Error Code
;		E3-E0	Error Name							
;		0001	Operation Incomplete (OPI)			
;		0010	Read Data CRC (DCRC) or Write Check Error (WCE)
;		0011	Header CRC (HCRC)					
;		0100	Data Late (DLT)						
;		0101	Header Not Found (HNF)				
;		1000	Non-Existent Memory (NXM)			
;		1001	Memory Parity Error (MPE) RLV12 only
;
;	14	Drive Error (DE) - This bit is tied directly to the DE interface line. 
;		When set, it indicates that the selected drive has flagged an error. 
;		(The source of the error can be determined by executing a get status 
;		command and then executing an MPR read.) DE can be cleared by executing 
;		a get status command with bit 3 of the DA register set.
;
;	15	Composite Error - When set, this bit indicates that one or more of the
;		error bits (bits 10-14) is set. If the IE bit (bit 6 of CS) is set and 
;		an error occurs (which sets bit 7), an interrupt will be initiated.
;
csrin:					; 36 cycles used when we reach 
	lds	qbuslow, CSRL		;  3
	lds	qbushigh, CSRH		;  3
	mov	zl, qbushigh		;  1
	andi	zl, driveselect		;  1
	swap	zl			;  1
	clr	zh			;  1
	subi	zl, low(-unittable)	;  1
	sbci	zh, high(-unittable)	;  1
	ldd	itemp, Z+ucb_status	;  2
	bst	itemp, ucb__rdy		;  1
	bld	qbuslow, CSR_DRVRDY	;  1
	bst	itemp, ucb__de		;  1
	bld	qbushigh, CSR_DE	;  1
	sts	CSRL, qbuslow		;  3
	sts	CSRH, qbushigh		;  3
	DATI	qbuslow, qbushigh	; 18
	DATI_exit qbuslow, qbushigh, 000; 40
					;---
					;118 cycles, 5.34usec

csrout:
;
;	Copy BA16 and BA17 and check whether the CRDY has been cleared
;
	DATO	qbuslow, qbushigh	; 16
	ldi	itemp, driveselect	;  1
	and	qbushigh, itemp		;  1
	lds	itemp, BAEL		;  3
	bst	qbuslow, CSR_BA16	;  1
	bld	itemp, BAE_BA16		;  1
	bst	qbuslow, CSR_BA17	;  1
	bld	itemp, BAE_BA17		;  1
	sts	BAEL, itemp		;  1
	sbrc	qbuslow, CSR_CRDY	;  1
	rjmp	csroutdone		;  2
	DATI	zero, zero		;   
	cbi	b_CRDY			;   
csroutdone:
	sts	CSRL, qbuslow		;  1
	sts	CSRH, qbushigh		;  1
	DATO_exit qbuslow, qbushigh, 000; 46
					;---
					;113 cycles, 5.11usec

;--------------------------------------------------------------------------
;
;	BAR	17774402
;
barin:
	lds	qbuslow, BARL
	lds	qbushigh, BARH
	DATI	qbuslow, qbushigh
	DATI_exit qbuslow, qbushigh, 002

barout:
	DATO	qbuslow, qbushigh
	sts	BARL, qbuslow
	sts	BARH, qbushigh
	DATO_exit qbuslow, qbushigh, 002
;--------------------------------------------------------------------------
;
;	DAR	17774404
;
;	DAR during seek
;
;	+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+
;	|DF8|DF7|DF6|DF5|DF4|DF3|DF2|DF1|DF0| 0 | 0 |HS | 0 |DIR| 0 | 1 |
;	+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+
;
;	DF		Difference in cylinder
;	HS		Head Select
;	DIR		Direction	0 move head towards spindle
;						1 move head towards boarder
;	DAR during read or write
;
;	+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+
;	|CA8|CA7|CA6|CA5|CA4|CA3|CA2|CA1|CA0|HS |SA5|SA4|SA3|SA2|SA1|SA0|
;	+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+
;
;	DAR during get status
;
;	+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+
;	| x | x | x | x | x | x | x | x | 0 | 0 | 0 | 0 |RST| 0 | 1 | 1 |
;	+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+
;
darin:
	lds	qbuslow, DARL
	lds	qbushigh, DARH
	DATI	qbuslow, qbushigh
	DATI_exit qbuslow, qbushigh, 004

darout:
	DATO	qbuslow, qbushigh
	sts	DARL, qbuslow
	sts	DARH, qbushigh
	DATO_exit qbuslow, qbushigh, 004

;--------------------------------------------------------------------------
;
;	MPR	17774406
;
;	The MPR register can have three meanings
;
;	MPR during get status
;
;	+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+
;	|WDE|CHE|WL |SKT|SPE|WGE|VC |DSE|DT |HS |CO |HO |BH |STC|STB|STA|
;	+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+
;
;	STA..C	Status
;	BH	Brush home
;	HO	Heads out
;	CO	Cover open
;	HS	Head select
;	DT	Drive Type
;	DSE	Drive Select Error
;	VC	Volume Check
;	WGE	Write Gate Error
;	SPE	Spin Error
;	SKTO	Seek Time-out
;	WL	Write Lock
;	CHE	Current Head Error
;	WDE	Write Data Error
;
;	MPR during read header
;
;	+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+
;	|CA8|CA7|CA6|CA5|CA4|CA3|CA2|CA1|CA0|HS |SA5|SA4|SA3|SA2|SA1|SA0|
;	+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+
;
;	+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+
;	| 0 | 0 | 0 | 0 | 0 | 0 | 0 | 0 | 0 | 0 | 0 | 0 | 0 | 0 | 0 | 0 |
;	+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+
;
;	+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+
;	|                              CRC                              |
;	+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+
;
;	MPR during write wordcount (write-only)
;
;	+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+
;	| 1 | 1 | 1 |        2's comlement of wordcount                 |
;	+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+
;
;
;	When reading MPR we actually read the FIFO
;
mprin:
	lds	qbuslow, MPR_Fifo	; Read top of FIFO
	lds	qbushigh, MPR_Fifo+1
	DATI	qbuslow, qbushigh
	lds	itemp, MPR_Fifo+2	; Shift Fifo, the real Fifo
	sts	MPR_Fifo+0, itemp	; on the RLV12 has 512 words
	lds	itemp, MPR_Fifo+3	; however we only need the
	sts	MPR_Fifo+1, itemp	; Fifo when we perform a read
	lds	itemp, MPR_Fifo+4	; header. 
	sts	MPR_Fifo+2, itemp	;
	lds	itemp, MPR_Fifo+5
	sts	MPR_Fifo+3, itemp
	lds	itemp, MPR_Fifo+6
	sts	MPR_Fifo+4, itemp
	lds	itemp, MPR_Fifo+7
	sts	MPR_Fifo+5, itemp
	DATI_exit qbuslow, qbushigh, 006
	
mprout:
	DATO	qbuslow, qbushigh
	sts	MPRL, qbuslow
	sts	MPRH, qbushigh
	DATO_exit qbuslow, qbushigh, 006

;--------------------------------------------------------------------------
;
;	BAE	17774410
;
;	Note: BA16 and BA17 are duplicated in BAE and CSR therefore 
;	whenever either register is written we update the other 
;	registers value of BA16 and BA17
;
baein:
	lds	qbuslow, BAEL
	lds	qbushigh, BAEH
	DATI	qbuslow, qbushigh
	DATI_exit qbuslow, qbushigh, 010

baeout:
	DATO	qbuslow, qbushigh	
	sts	BAEL, qbuslow
	sts	BAEH, qbushigh		;;; This might update BA16, BA17 so we need to
	lds	itemp, CSRL		;;; update BA16 and BA17 in CSR 
	bst	qbuslow, BAE_BA16
	bld	itemp, CSR_BA16
	bst	qbuslow, BAE_BA17
	bld	itemp, CSR_BA17
	sts	CSRL, itemp
	DATO_exit qbuslow, qbushigh, 010

;--------------------------------------------------------------------------
;
;	BAE2	17774412
;
bae2in:
	lds	qbuslow, BAEL+2
	lds	qbushigh, BAEH+2
	DATI	qbuslow, qbushigh
	DATI_exit qbuslow, qbushigh, 012

bae2out:
	DATO	qbuslow, qbushigh
	sts	BAEL+2, qbuslow
	sts	BAEH+2, qbushigh
	DATO_exit qbuslow, qbushigh, 012
;--------------------------------------------------------------------------
;
;	BAE4	17774414
;
;	This register is not implemented in the real RLV12 controller
;	but many 3rd party controllers make use of these registers.
;	The Plessey SASI interface that emulates an RLV12 controller
;	for example implements a boot feature when the CPU executes
;	the instruction at 174414 in the IO-page. The trick is very 
;	nice. Whenever the CPU fetches an instuction the controller
;	will either return 0777, a branch to itslef, or 05001 which
;	just clears R1 to make sure the boot block assumes that we
;	are booting unit 0. When the controller returns the branch
;	to itself instruction it will at the same time initiate a
;	DMA of a branch to itself instruction to the physical memory
;	address 0. Once the DMA has finished the next time the processor
;	fetches an instuction from 174414 the controller returns the
;	CLR R0 instruction, which causes the processor to fetch the
;	next instruction from 174416. Reading 174416 however will
;	always return the value 05007, which is the CLR PC instruction.
;	This effectively is a JMP to the physical address 0. When 
;	this happens the controller will execute the autoboot function
;	and the processor loops at address 0 because of the instruction
;	the controller has placed there.
;	Autoboot will read block 0 from unit 0 and write the block
;	starting with Word 1 to physical address 2 using DMA. Once
;	this has been completed the controller then writes Word 0
;	from block 0 to physical address 0 which then causes the
;	processor to execute the boot code in block 0 of unit 0 and
;	hence will continue to boot unit 0.
;
bae4in:					; 36
	ldi	zl, low(0777)		;  1 Assume DMA still pending
	ldi	zh, high(0777)		;  1
	sbis	b_DMR			;  1 did we already request DMA?
	rjmp	bae4in_dma		;  2 no do it now
	sbis	i_DMG			;  1 did it finish?
	rjmp	bae4in_cont		;  2 no continue to send BR .
	cbi	b_DMR			;  2 remove DMA request
	ldi	zl, low(05001)		;  1 DMA finished return a CLR R1
	ldi	zh, high(05001)		;  1
        sts	autoboot, ff		;  2 Requst Auto Boot !!!!!!
bae4in_cont:
	DATI	zl, zh			; 18
	movw	qbushigh:qbuslow, zh:zl	;  2
	DATI_exit qbuslow, qbushigh, 014; 40
					;---
					;110
;
;	The first time we read BOOT4 we start a DMA to transfer BR .
;	instruction to adddress zero and as well return a BR . instruction.
;	
bae4in_dma:				; 41
	cbi	b_RD			;  2
	out	dataportdir, ff		;  1
	ldi	itemp, 4		;  1
	out	dataportout, itemp	;  1
	sbi	b_ALEW			;  2
	cbi	b_ALEW			;  2
	out	dataportout, zero	;  1
	sbi	b_WR			;  2
	cbi	b_WR			;  2
	out	dataportout, zero	;  1
	sbi	b_WR			;  2
	cbi	b_WR			;  2
	out	dataportout, zero	;  1
	sbi	b_WR			;  2
	cbi	b_WR			;  2
	ldi	itemp, 2		;  1
	out	dataportout, itemp	;  1
	sbi	b_ALEW			;  2
	cbi	b_ALEW			;  2
	out	dataportout, zl		;  1
	sbi	b_WR			;  2
	cbi	b_WR			;  2
	out	dataportout, zh		;  1
	sbi	b_WR			;  2
	cbi	b_WR			;  2
	sbi	b_DMR			;  2
	DATI	zl, zh			; 18
	movw	qbushigh:qbuslow, zh:zl	;  2
	DATI_exit qbuslow, qbushigh, 014; 40
					;---
					;143 cycles, 6.47usec

bae4out:
	DATO	qbuslow, qbushigh
	sts	BAEL+4, qbuslow
	sts	BAEH+4, qbushigh
	DATO_exit qbuslow, qbushigh, 014
;--------------------------------------------------------------------------
;
;	BAE2	17774416
;
bae6in:
	ldi	zl, low(05007)
	ldi	zh, high(05007)
	DATI	zl, zh
	movw	qbushigh:qbuslow, zh:zl
	DATI_exit qbuslow, qbushigh, 016

bae6out:
	DATO	qbuslow, qbushigh
	sts	BAEL+6, qbuslow
	sts	BAEH+6, qbushigh
	DATO_exit qbuslow, qbushigh, 016

;--------------------------------------------------------------------------
;
;	Process DATI/DATO cycles to the ROM range 173xxx in IO-page
;
INTQ_ROM:
	.if	signal==1
	cbi	b_SIG
	nop
	sbi	b_SIG			;;;
	.endif
	bst	zl, WTBT		;;; put to T-bit
	brts	INTQ_DATO		;;; 
	sbi	b_ALER			;;; Advance to register 3
	cbi	b_ALER
	sbi	b_ALER
	cbi	b_ALER
	sbi	b_ALER
	cbi	b_ALER
	nop
	nop
	nop
	nop
	in	itemp, dataportin	;;; ROM Address
	mov	zl, itemp
	clr	zh
	add	zl, zl
	adc	zh, zh
	subi	zl, low(-softrombuffer)
	sbci	zh, high(-softrombuffer)
	ld	qbuslow, Z+
	ld	qbushigh, Z+
	DATI	qbuslow, qbushigh
	rjmp	INTQ_EXIT

INTQ_DATO:
	lds	zl, romstatus
	sbrc	zl, 1			;;; RO bit set?
	rjmp	INTQ_NOLOG		;;; do nothing
	DATO	qbuslow, qbushigh
	sbi	b_ALER
	cbi	b_ALER
	nop
	nop
	nop
	in	itemp, dataportin	;; ROM Address
;
;	Writeable Soft ROM
;	
	mov	zl, itemp
	clr	zh
	add	zl, zl
	adc	zh, zh
	subi	zl, low(-softrombuffer)
	sbci	zh, high(-softrombuffer)
	st	Z+, qbuslow
	st	Z+, qbushigh
	
INTQ_EXIT:
	sbis	GPIOR0, log__rom
	rjmp	INTQ_NOLOG
	lds	zl, log_pointer+0	;;; Logging
	lds	zh, log_pointer+1	;;; Logging
	std	Z+0, one		;;; 1	= Valid Log Entry with FNC=Maint
	std	Z+2, qbuslow		;;; low byte
	std	Z+3, qbushigh		;;; high byte
	std	Z+4, itemp		;;; ROM address
	ldi	itemp, log_romread	;;; 4 = Read ROM
	bld	itemp, 0		;;; 5 = Write ROM
	std	Z+1, itemp		;;; 
	lds	itemp, TCNT2		;;; Get ticks
	std	Z+5, itemp		;;; Store ticks
	lognxt	zl, zh			;;; next address address
INTQ_NOLOG:
	sbi	b_IACK
	cbi	b_IACK
	pop	zh			;;; INT EXIT
	pop	zl
	out	SREG, isreg
	.if	signal==1
	cbi	b_SIG
	.endif
	reti

;--------------------------------------------------------------------------
;
;	INTI is the IACK cycle interrupt. IACK is now implemented as a MCU
;	interrupt and the CPLD expects now the MCU to set the vector value.
;	If needed the interrupt can also do some processing on the device
;	registers, e.g. the RH70 massbus controller clears the IE bit when
;	the PDP-11 performs an IACK cycle. Other controllers, like MSCP
;	controllers, require a programmable interrupt vector.
;
INTI_isr:
	.if	signal==1
	sbi	b_SIG			;Pulse
	nop
	cbi	b_SIG
	nop
	sbi	b_SIG			;Pulse
	nop
	cbi	b_SIG
	nop
	sbi	b_SIG			;Signal
	.endif
	cbi	b_IRQST			;;; De-assert IRQ
	in	isreg, SREG		;;; Save CPU status register
	push	zl			;;;
	push	zh			;;;
	ldi	zl, low(RLVVECTOR)	;;; load interrupt vector address
	ldi	zh, high(RLVVECTOR)	;;;
	DATI	zl, zh			;;; return value to Q-Bus

	.if debug==1
	lds	itemp, INTI_count	;;; Update interrupt counter
	inc	itemp			;;;
	sts	INTI_count, itemp	;;;
	.endif

	sbis	GPIOR0, log__iack	;;; check logging of interrupts
	rjmp	INTI_nologg
	lds	zl, log_pointer+0	;;; Logging
	lds	zh, log_pointer+1	;;; Logging
	std	Z+0, one		;;; 1	= Valid Log Entry with FNC=Maint
	ldi	itemp, log_iack		;;; 3	= IACK
	std	Z+1, itemp		;;;
	ldi	itemp, low(0160)	;;; Default Vector for RLV12
	std	Z+2, itemp		;;; 
	ldi	itemp, high(0160)	;;; Default Vector for RLV12
	std	Z+3, itemp		;;; 
	lds	itemp, TCNT2		;;; Get ticks
	std	Z+5, itemp		;;; Store ticks
	lognxt	zl, zh			;;; next address address
INTI_nologg:

	pop	zh
	pop	zl
	out	SREG, isreg
	sbi	b_IACK			;;; Acknowledge CPLD interrupt
	cbi	b_IACK
	.if	signal==1
	cbi	b_SIG
	.endif
	reti
;--------------------------------------------------------------------------
;
;	INIT is the BINIT interrupt. We will receive an interrupt when BINIT
;	is either asserted or de-asserted. 
;
;	INIT will bring the RLV12 interface to a known state. It will reset
;	all RLV12 registers and clear the units. However it will keep all
;	assignements and attached partitions and files.
;
;	It will reset all CPLD states and release IRQ, DMA, etc. and sets 
;	the outpus to a known state
;
;	IO setup, Watch Dog, Volume status etc will be kept. 
;
;	Note: BINIT will be asserted by a PDP-11 RESET instruction, so
;	this interrupt is supposed to have the same effect as has a normal
;	reset on the real RLV12 controller.
;
;	This is now implemented as a Pin Change Interrupt. This allows us
;	to perform different actions when BINIT is asserted and when
;	BINIT is deasserted. Note however that when BINIT is asserted we
;	have only about 60usec before BINIT is asserted again, which still
;	is about 1000 instructions at 22MHz.
;
PC_INIT:
	push	r8
	in	r8, SREG
	push	r16
	push	zl
	push	zh

	.if debug==1
	lds	r16, INIT_count
	inc	r16
	sts	INIT_count, r16
	.endif
	
	sbis	GPIOR0, log__iack
	rjmp	PC_INIT_nologg
	lds	zl, log_pointer+0	;;; Logging
	lds	zh, log_pointer+1	;;; Logging
	ldi	r16, 1			;;; 
	std	Z+0, r16		;;; 1	= Valid Log Entry with FNC=Maint
	ldi	r16, log_init		;;; 7	= BINIT
	std	Z+1, r16		;;;
	
	ldi	r16, 0
	sbic	i_INIT
	ldi	r16, 1
	std	Z+2, r16		;;; Store Flag
	std	Z+3, r16		;;; 
	lds	r16, TCNT2		;;; Get ticks
	std	Z+5, r16		;;; Store ticks
	lognxt	zl, zh			;;; next address address
PC_INIT_nologg:

	cbi	b_ALEW
	cbi	b_ALER
	cbi	b_RD
	cbi	b_WR
	cbi	b_LED
	sts	led_oneshot, zero
	cbi	b_IRQST
	cbi	b_DMR
	sbi	b_IACK
	cbi	b_IACK
	sbi	b_ABORT
	cbi	b_ABORT
	call	rlv12_reset
	pop	zh
	pop	zl
	pop	r16
	out	SREG, r8
	pop	r8
	reti	

