;--------------------------------------------------------------------------
;
;	Each controller supports a number of units. Most controller support
;	either 4 or 8 units. For each unit we keep a unit control block in
;	a contiguous memory area. To allow for fast translation of unit number
;	to unit control block address we make each unit control either 8, 16
;	or 32 bytes long (we should avoid 64 bytes although the ld and st with
;	displacement support an offset from 0 to 63). Each unit control block
;	has a header of 4 bytes which is controller independant.
;
;	Units can be either attached to partitions on the SD-Card or to
;	files on the FAT-32 volume on the SD-Card.
;
;	Status contains the status bits of which 3 are controller agnostic
;	DL_DRDY		drive is ready, typically this is when the drive
;			is attached
;	DL_PART		drive is attached to a partition
;	DL_FILE		drive is attached to a file
;	Type is the byte copied over from the drive table the use of the
;	field is optional, that is the controller may use it or not
;	RLV12		bit 0 is cleared for a RL01 and set for a RL02
;	RH70		contains the drive code read via RMDT
;	ImgPtr is the pointer either to the partition control block or to
;	the file control block
;
;	Offset definitions unit control blocks
;
recordstart	ucb
record		ucb, status, 1
record		ucb, type, 1
record		ucb, imgptr, 2
record		ucb, diskaddr, 2
recordend	ucb, size
;.equ	ucb_status	= 0		; Volume Status
;.equ	ucb_type	= 1		; Volume Type
;.equ	ucb_imgptr	= 2 		; Pointer to "Image"
;
;	Controller specific definitions
;
;.equ	DL_DAR		= 4		; Last DAR requested 
.equ	DL_SeekCmd	= 6 		; Seek Command per drive
;
;
;	Bit defintions of ucb_status
;
.equ	ucb__rdy	= 0		; Drive is ready
.equ	ucb__part	= 1		; Volume is mapped to a file
.equ	ucb__file	= 2		; Volume is mapped to a partition
.equ	ucb__de		= 3		; Drive Error
.equ	ucb__log	= 7		; Drive Logging
;
;	Controller specific bit definitions
;
.equ	DL_DRSEEK	= 3		; Drive performs a seek
;
;	Controller specific defintions of ucb_type
;
.equ	DL_RL02		= 0		; Drive is RL02

;
;	RLV12 Register bit definitions
;
.equ	CSR_DS1		= 1
.equ	CSR_DS0		= 0

.equ	CSR_ERR		= 7
.equ	CSR_DE		= 6

.equ	CSR_CRDY	= 7
.equ	CSR_IE		= 6
.equ	CSR_BA17	= 5
.equ	CSR_BA16	= 4
.equ	CSR_FC2		= 3
.equ	CSR_FC1		= 2
.equ	CSR_FC0		= 1
.equ	CSR_DRVRDY	= 0

.equ	BAE_BA17	= 1
.equ	BAE_BA16	= 0

.equ	DAR_SEEK_DIR	= 2		; Direction bit in DAR for seek
.equ	DAR_SEEK_HS	= 4		; Head select bit in DAR for seek
.equ	DAR_RW_HS	= 6		; Head select bin in DAR for read/write
.equ	MPR_GETS_HS	= 6		; Head select bin in MPR for get status
.equ	MPR_GETS_DT	= 7		; Drive Type bit in MPR for get status


#define driveselect (1<<CSR_DS1) | (1<<CSR_DS0)
#define fuctioncode (1<<CSR_FC2) | (1<<CSR_FC1) | (1<<CSR_FC0)
;
;	RLV12 CSR register storage area
;
		.dseg
		align	4
CSRL:		.byte	1
CSRH:		.byte	1
BARL:		.byte	1
BARH:		.byte	1
DARL:		.byte	1
DARH:		.byte	1
MPRL:		.byte	1
MPRH:		.byte	1
BAEL:		.byte	1
BAEH:		.byte	1
		.byte	6		; not implemented but addressable
;
;	MPR Fifo for Read Header
;
MPR_Fifo:	.byte	8		; MPR Fifo buffer

;
;	After scanning the partitions the first 4 cartridges found will be mapped
;	to active volumes. If required we can copy the cartridge information to
;	the respective volume to "change" the cartridge.
;
		align	5
.equ	units		= 4
.equ	unitsize	= 16
unittable:	.byte	units * unitsize
