;--------------------------------------------------------------------------
;	
;	Input:
;
;	Y pointer to standardised parameter block
;
;	Read block rom SD-Card
;
;	Registers:
;
;	None
;
;	Timed SPI SD_Card_Read for highest performance
;
SD_CARD_READ:
	push	r16
	push	r17
	push	r18
	push	r19
	push	r24
	push	r25
	push	xl
	push	xh
	push	zl
	push	zh

	ldi	r24, (0<<SPIE)|(1<<SPE)|(0<<DORD)|(1<<MSTR)|(0<<CPOL)|(0<<CPHA)|(0<<SPR1)|(0<<SPR0)
	out	SPCR, r24
;
	ldi	r24, (spi2xb<<SPI2X)	; Requires a pull-up on MISO!!!
	out	SPSR, r24		; Initialise SPI interface
;
	ldi	r24, 0x51		; Read Sector
	sts	sd_cmd+0, r24
	rcall	SD_CARD_XLT		; Translate Sector to SD Card address
	ldi	r24, 0xff
	sts	sd_cmd+5, r24
	rcall	SD_SEND_CMD
	rcall	SPI_GET_RESPONSE
	std	Y+P_IOStatus, r24
	tst	r24
	breq	SD_READ000
	rjmp	SD_READ910

SD_READ000:
	;+	Start Timer
	sts	TCCR1B, zero		; Stop
	in	r24, TIFR1
	out	TIFR1, r24		; Reset Flags
	sts	TCNT1H, zero
	sts	TCNT1L, zero
	sts	TCCR1A, zero
	ldi	r24, (1<<CS12) | (0<<CS11) | (0<<CS10)	; Prescaler 256 -> 1 r24 ~10usec
	sts	TCCR1B, r24		; Start
	;-	Start Timer

SD_READ010:
	ldi	r24, 0xff
	out	SPDR, r24
SD_READ020:
	in	r24, SPSR
	sbrs	r24, SPIF
	rjmp	SD_READ020
	in	r24, SPDR
	cpi	r24, 0xfe		; Start data token?
	breq	SD_READ030
	cpi	r24, 0xff		; Wait data token?
	breq	SD_READ010
	ldi	r24, 1
	std	Y+P_IOStatus, r24	; Error 1 invalid token received
	rjmp	SD_READ910
;
;	Timed Read, assuming SPI Clock = CPU Clock / 4 
;	After writing a dummy byte to SPDR, 32 cycles must pass before
;	we can read SPDR to get the byte from the SD-Card, that is for
;	reads the cycle must be exactly 33cycles
;
SD_READ030:
	ldi	r17, 0xff		; r17 is now used as value 0xFF !!!
	out	SPDR, r17
	ldi	r24, low(512)		;  1
	ldi	r25, high(512)		;  1
	ldd	xl, Y+P_Address+0	;  2
	ldd	xh, Y+P_Address+1	;  2
	clr	r18			;  1
	clr	r19			;  1	->  8
	
	.if	spi2xb == 0
	rjmp	PC+1			;  2
	rjmp	PC+1			;  2
	rjmp	PC+1			;  2
	rjmp	PC+1			;  2	->  8

	rjmp	PC+1			;  2
	rjmp	PC+1			;  2
	rjmp	PC+1			;  2
	rjmp	PC+1			;  2	->  8
	.endif

	rjmp	PC+1			;  2
	rjmp	PC+1			;  2
	rjmp	PC+1			;  2
	rjmp	PC+1			;  2	->  8 =>32cycles have past since "out"
;
;	Timed SPI read cycle when SPI CLK is 1/4 of system clock a SPI byte transfer
;	takes 32 clock cycles. One extra cycle is required before we start the next
;	read, therefore the complete read byte loop must take 33 or more cycle.
;
;	When SP CLK is 1/2 of system clock then the read byte loop must take 17 cycles.
;
SD_READ040:
	in	r16, SPDR		;  1
	out	SPDR, r17		;  1
	st	X+, r16		;  2
	crc	r16, r18, r19		; 11	-> 15
	
	.if	spi2xb == 0
	rjmp	PC+1			;  2	-> 17

	rjmp	PC+1			;  2
	rjmp	PC+1			;  2
	rjmp	PC+1			;  2
	rjmp	PC+1			;  2	->  8

	rjmp	PC+1			;  2
	rjmp	PC+1			;  2
	.endif
	sbiw	r25:r24, 1		;  2
	brne	SD_READ040		;  2	->  8 => 33cycles for the loop
	nop				;  1 	compensate brne not taken

	in	r24, SPDR		;  
	out	SPDR, r17		;  
	std	Y+P_IOStatus+0, r24	;  2
	rjmp	PC+1			;  2
	rjmp	PC+1			;  2
	rjmp	PC+1			;  2	->  8
	
	.if	spi2xb == 0
	rjmp	PC+1			;  2
	rjmp	PC+1			;  2
	rjmp	PC+1			;  2
	rjmp	PC+1			;  2	->  8

	rjmp	PC+1			;  2
	rjmp	PC+1			;  2
	rjmp	PC+1			;  2
	rjmp	PC+1			;  2	->  8
	.endif
	rjmp	PC+1			;  2
	rjmp	PC+1			;  2
	rjmp	PC+1			;  2
	rjmp	PC+1			;  2	->  8 => 32
	
	in	r24, SPDR
	std	Y+P_IOStatus+1, r24
	std	Y+P_IOStatus+2, r18
	std	Y+P_IOStatus+3, r19
	rcall	SPI_CLEAR
	clc
	rjmp	SD_READ_SUCCESS
	
SD_READ910:
	rcall	SPI_CLEAR
	sec
SD_READ_SUCCESS:
	;+	Read Timer
	sts	TCCR1B, zero		; Stop
	ldi	r17, 0xff
	std	Y+P_Duration+0, r17
	std	Y+P_Duration+1, r17	; Assume overflow
	sbic	TIFR1, TOV1		; Overflow set?	
	rjmp	SD_READ_SUCCESS010	; Yes, so just report maximum
	lds	r24, TCNT1L		; Get Counter
	std	Y+P_Duration+0, r24	;
	lds	r24, TCNT1H
	std	Y+P_Duration+1, r24	; Save number of cycles before read data
SD_READ_SUCCESS010:
	;-	Read Timer
	pop	zh
	pop	zl
	pop	xh
	pop	xl
	pop	r25
	pop	r24
	pop	r19
	pop	r18
	pop	r17
	pop	r16
	ret
;--------------------------------------------------------------------------
;	
;	Input:	Y pointer to standardised parameter block
;
;	Write block to SD-Card
;
SD_CARD_WRITE:
	push	xl
	push	xh
	push	zl
	push	zh

	;+	Start Timer
	sts	TCCR1B, zero		; Stop
	in	r24, TIFR1
	out	TIFR1, r24		; Reset Flags
	sts	TCNT1H, zero
	sts	TCNT1L, zero
	sts	TCCR1A, zero
	ldi	r24, (1<<CS12) | (0<<CS11) | (0<<CS10)	; Prescaler 256 -> 1 r24 ~10usec
	sts	TCCR1B, r24		; Start
	;-	Start Timer
	
	ldi	r24, (0<<SPIE)|(1<<SPE)|(0<<DORD)|(1<<MSTR)|(0<<CPOL)|(0<<CPHA)|(0<<SPR1)|(0<<SPR0)
	out	SPCR, r24
;
	ldi	r24, (0<<SPI2X)		; Write uses slow clock
	out	SPSR, r24		; Initialise SPI interface
;
	ldi	r24, 0x58		; Write Sector
	sts	sd_cmd+0, r24
	rcall	SD_CARD_XLT		; Translate Sector to SD-Card address
	ldi	r24, 0xff
	sts	sd_cmd+5, r24
	rcall	SD_SEND_CMD		; Send Write Sector Command
	rcall	SPI_GET_RESPONSE	; Get Return status
	std	Y+P_IOStatus, r24		; Save it
	tst	r24			; Ok?
	brne	SD_CARD_WRITE_ERR1	; No error
	
	ldi	r24, 0xfe		; Data token
	rcall	SPI_WRITE_BYTE		; Write
	ldi	zl, low(512)		; Prepare pointer
	ldi	zh, high(512)
	ldd	xl, Y+P_Address+0
	ldd	xh, Y+P_Address+1
SD_CARD_WRITE_L1:			; 
	ld	r24, X+			; Get Byte
	rcall	SPI_WRITE_BYTE		; Write
	sbiw	zh:zl, 1
	brne	SD_CARD_WRITE_L1	; 512bytes
	
	ldi	r24, 0x01		; Dummy CRC
	rcall	SPI_WRITE_BYTE
	ldi	r24, 0x01
	rcall	SPI_WRITE_BYTE

	rcall	SPI_WRITE_DUMMY		; Get status
	in	r24, SPDR		;
	std	Y+P_IOStatus+1, r24	; Save in status block
	andi	r24, 0x1f
	cpi	r24, 0x05		; must be 0x05 mod 0x1f
	breq	SD_CARD_WRITE_SUCC	; yes success
SD_CARD_WRITE_ERR1:	
	rcall	SPI_CLEAR		; Stop SPI
	sec				; Error
	rjmp	SD_CARD_WRITE_DONE

SD_CARD_WRITE_SUCC:	
	rcall	SPI_WRITE_DUMMY		; Until the SD-Card has finished
	in	r24, SPDR		; internal operation it will pull
	tst	r24			; MISO low, during that time
	breq	SD_CARD_WRITE_SUCC	; you must not send another command
	rcall	SPI_CLEAR		; Stop SPI
	clc				; Success
SD_CARD_WRITE_DONE:	
	;+	Read Timer
	sts	TCCR1B, zero		; Stop
	ldi	r17, 0xff
	std	Y+P_Duration+0, r17
	std	Y+P_Duration+1, r17	; Assume overflow
	sbis	TIFR1, TOV1
	rjmp	SD_CARD_WRITE_DONE010
	lds	r24, TCNT1L
	std	Y+P_Duration+0, r24
	lds	r24, TCNT1H
	std	Y+P_Duration+1, r24	; Save number of cycles before read data
SD_CARD_WRITE_DONE010:	
	;-	Read Timer

	pop	zh
	pop	zl
	pop	xh
	pop	xl
	ret
;--------------------------------------------------------------------------
;
;	Translate Sector in SD-Card address
;
SD_CARD_XLT:
	lds	r24, sd_status
	andi	r24, sd__ccs_bm
	brne	SD_CARD_XLTHC		; HCXC Card
	sts	sd_cmd+4, zero		; Standard SD Cards

	ldd	r24, Y+P_Sector+0	; require a byte offset so we
	add	r24, r24		; need to translate the sector
	sts	sd_cmd+3, r24		; to a byte offset, as block length

	ldd	r24, Y+P_Sector+1	; has been set to 512 this is just
	adc	r24, r24		; writing twice the sector number in
	sts	sd_cmd+2, r24		; psector to one byte offset of the

	ldd	r24, Y+P_Sector+2	; byte offset field of the read/write
	adc	r24, r24		; command
	sts	sd_cmd+1, r24

	ret
SD_CARD_XLTHC:				; HCXC Card expect just the sector
	ldd	r24, Y+P_Sector+0	; Number.
	sts	sd_cmd+4, r24
	ldd	r24, Y+P_Sector+1
	sts	sd_cmd+3, r24
	ldd	r24, Y+P_Sector+2
	sts	sd_cmd+2, r24
	ldd	r24, Y+P_Sector+3
	sts	sd_cmd+1, r24
	ret	
;--------------------------------------------------------------------------
;
;	Calculate CRC-7 left adjusted as required by SD-Card
;
SD_commandCRC:
; uint8_t SD_commandCRC(uint8_t *cmd)
	ldi	xl, low(sd_cmd)
	ldi	xh, high(sd_cmd)
;	movw	xh:xl, r25:r24
	ldi	r24, 0x40		; Initiate CRC with start of Bit7=0, Bit6=1
	ldi	r16, 5			; 5 bytes
SD_commandCRC010:
	ld	r17, X+			; Get Next Command byte
	eor	r24, r17		; xor
	mov	zl, r24			; make index
	ldi	zh, high(2*crc7table)	; translate
	lpm	r24, Z			; new CRC
	dec	r16
	brne	SD_commandCRC010
	sts	sd_cmd+5, r24
	ret				; r24 contains CRC7 left-adjusted
;=========================================================================
;
;	Note the updcrc macro assumes that the tables are page aligned
;
;	low byte CRC lookup table
; 
	align	8
crclo:
 .db 0x00,0x21,0x42,0x63,0x84,0xA5,0xC6,0xE7,0x08,0x29,0x4A,0x6B,0x8C,0xAD,0xCE,0xEF
 .db 0x31,0x10,0x73,0x52,0xB5,0x94,0xF7,0xD6,0x39,0x18,0x7B,0x5A,0xBD,0x9C,0xFF,0xDE
 .db 0x62,0x43,0x20,0x01,0xE6,0xC7,0xA4,0x85,0x6A,0x4B,0x28,0x09,0xEE,0xCF,0xAC,0x8D
 .db 0x53,0x72,0x11,0x30,0xD7,0xF6,0x95,0xB4,0x5B,0x7A,0x19,0x38,0xDF,0xFE,0x9D,0xBC
 .db 0xC4,0xE5,0x86,0xA7,0x40,0x61,0x02,0x23,0xCC,0xED,0x8E,0xAF,0x48,0x69,0x0A,0x2B
 .db 0xF5,0xD4,0xB7,0x96,0x71,0x50,0x33,0x12,0xFD,0xDC,0xBF,0x9E,0x79,0x58,0x3B,0x1A
 .db 0xA6,0x87,0xE4,0xC5,0x22,0x03,0x60,0x41,0xAE,0x8F,0xEC,0xCD,0x2A,0x0B,0x68,0x49
 .db 0x97,0xB6,0xD5,0xF4,0x13,0x32,0x51,0x70,0x9F,0xBE,0xDD,0xFC,0x1B,0x3A,0x59,0x78
 .db 0x88,0xA9,0xCA,0xEB,0x0C,0x2D,0x4E,0x6F,0x80,0xA1,0xC2,0xE3,0x04,0x25,0x46,0x67
 .db 0xB9,0x98,0xFB,0xDA,0x3D,0x1C,0x7F,0x5E,0xB1,0x90,0xF3,0xD2,0x35,0x14,0x77,0x56
 .db 0xEA,0xCB,0xA8,0x89,0x6E,0x4F,0x2C,0x0D,0xE2,0xC3,0xA0,0x81,0x66,0x47,0x24,0x05
 .db 0xDB,0xFA,0x99,0xB8,0x5F,0x7E,0x1D,0x3C,0xD3,0xF2,0x91,0xB0,0x57,0x76,0x15,0x34
 .db 0x4C,0x6D,0x0E,0x2F,0xC8,0xE9,0x8A,0xAB,0x44,0x65,0x06,0x27,0xC0,0xE1,0x82,0xA3
 .db 0x7D,0x5C,0x3F,0x1E,0xF9,0xD8,0xBB,0x9A,0x75,0x54,0x37,0x16,0xF1,0xD0,0xB3,0x92
 .db 0x2E,0x0F,0x6C,0x4D,0xAA,0x8B,0xE8,0xC9,0x26,0x07,0x64,0x45,0xA2,0x83,0xE0,0xC1
 .db 0x1F,0x3E,0x5D,0x7C,0x9B,0xBA,0xD9,0xF8,0x17,0x36,0x55,0x74,0x93,0xB2,0xD1,0xF0 
;
;	hi byte CRC lookup table
;
crchi:
 .db 0x00,0x10,0x20,0x30,0x40,0x50,0x60,0x70,0x81,0x91,0xA1,0xB1,0xC1,0xD1,0xE1,0xF1
 .db 0x12,0x02,0x32,0x22,0x52,0x42,0x72,0x62,0x93,0x83,0xB3,0xA3,0xD3,0xC3,0xF3,0xE3
 .db 0x24,0x34,0x04,0x14,0x64,0x74,0x44,0x54,0xA5,0xB5,0x85,0x95,0xE5,0xF5,0xC5,0xD5
 .db 0x36,0x26,0x16,0x06,0x76,0x66,0x56,0x46,0xB7,0xA7,0x97,0x87,0xF7,0xE7,0xD7,0xC7
 .db 0x48,0x58,0x68,0x78,0x08,0x18,0x28,0x38,0xC9,0xD9,0xE9,0xF9,0x89,0x99,0xA9,0xB9
 .db 0x5A,0x4A,0x7A,0x6A,0x1A,0x0A,0x3A,0x2A,0xDB,0xCB,0xFB,0xEB,0x9B,0x8B,0xBB,0xAB
 .db 0x6C,0x7C,0x4C,0x5C,0x2C,0x3C,0x0C,0x1C,0xED,0xFD,0xCD,0xDD,0xAD,0xBD,0x8D,0x9D
 .db 0x7E,0x6E,0x5E,0x4E,0x3E,0x2E,0x1E,0x0E,0xFF,0xEF,0xDF,0xCF,0xBF,0xAF,0x9F,0x8F
 .db 0x91,0x81,0xB1,0xA1,0xD1,0xC1,0xF1,0xE1,0x10,0x00,0x30,0x20,0x50,0x40,0x70,0x60
 .db 0x83,0x93,0xA3,0xB3,0xC3,0xD3,0xE3,0xF3,0x02,0x12,0x22,0x32,0x42,0x52,0x62,0x72
 .db 0xB5,0xA5,0x95,0x85,0xF5,0xE5,0xD5,0xC5,0x34,0x24,0x14,0x04,0x74,0x64,0x54,0x44
 .db 0xA7,0xB7,0x87,0x97,0xE7,0xF7,0xC7,0xD7,0x26,0x36,0x06,0x16,0x66,0x76,0x46,0x56
 .db 0xD9,0xC9,0xF9,0xE9,0x99,0x89,0xB9,0xA9,0x58,0x48,0x78,0x68,0x18,0x08,0x38,0x28
 .db 0xCB,0xDB,0xEB,0xFB,0x8B,0x9B,0xAB,0xBB,0x4A,0x5A,0x6A,0x7A,0x0A,0x1A,0x2A,0x3A
 .db 0xFD,0xED,0xDD,0xCD,0xBD,0xAD,0x9D,0x8D,0x7C,0x6C,0x5C,0x4C,0x3C,0x2C,0x1C,0x0C
 .db 0xEF,0xFF,0xCF,0xDF,0xAF,0xBF,0x8F,0x9F,0x6E,0x7E,0x4E,0x5E,0x2E,0x3E,0x0E,0x1E 
;=============================================================================
;
;	Page aligned CRC tables
;
;	CRC-7 left adjusted https://github.com/spotify/linux/blob/master/lib/crc7.c
;
crc7table:
 .db 0x00,0x12,0x24,0x36,0x48,0x5a,0x6c,0x7e,0x90,0x82,0xb4,0xa6,0xd8,0xca,0xfc,0xee
 .db 0x32,0x20,0x16,0x04,0x7a,0x68,0x5e,0x4c,0xa2,0xb0,0x86,0x94,0xea,0xf8,0xce,0xdc
 .db 0x64,0x76,0x40,0x52,0x2c,0x3e,0x08,0x1a,0xf4,0xe6,0xd0,0xc2,0xbc,0xae,0x98,0x8a
 .db 0x56,0x44,0x72,0x60,0x1e,0x0c,0x3a,0x28,0xc6,0xd4,0xe2,0xf0,0x8e,0x9c,0xaa,0xb8
 .db 0xc8,0xda,0xec,0xfe,0x80,0x92,0xa4,0xb6,0x58,0x4a,0x7c,0x6e,0x10,0x02,0x34,0x26
 .db 0xfa,0xe8,0xde,0xcc,0xb2,0xa0,0x96,0x84,0x6a,0x78,0x4e,0x5c,0x22,0x30,0x06,0x14
 .db 0xac,0xbe,0x88,0x9a,0xe4,0xf6,0xc0,0xd2,0x3c,0x2e,0x18,0x0a,0x74,0x66,0x50,0x42
 .db 0x9e,0x8c,0xba,0xa8,0xd6,0xc4,0xf2,0xe0,0x0e,0x1c,0x2a,0x38,0x46,0x54,0x62,0x70
 .db 0x82,0x90,0xa6,0xb4,0xca,0xd8,0xee,0xfc,0x12,0x00,0x36,0x24,0x5a,0x48,0x7e,0x6c
 .db 0xb0,0xa2,0x94,0x86,0xf8,0xea,0xdc,0xce,0x20,0x32,0x04,0x16,0x68,0x7a,0x4c,0x5e
 .db 0xe6,0xf4,0xc2,0xd0,0xae,0xbc,0x8a,0x98,0x76,0x64,0x52,0x40,0x3e,0x2c,0x1a,0x08
 .db 0xd4,0xc6,0xf0,0xe2,0x9c,0x8e,0xb8,0xaa,0x44,0x56,0x60,0x72,0x0c,0x1e,0x28,0x3a
 .db 0x4a,0x58,0x6e,0x7c,0x02,0x10,0x26,0x34,0xda,0xc8,0xfe,0xec,0x92,0x80,0xb6,0xa4
 .db 0x78,0x6a,0x5c,0x4e,0x30,0x22,0x14,0x06,0xe8,0xfa,0xcc,0xde,0xa0,0xb2,0x84,0x96
 .db 0x2e,0x3c,0x0a,0x18,0x66,0x74,0x42,0x50,0xbe,0xac,0x9a,0x88,0xf6,0xe4,0xd2,0xc0
 .db 0x1c,0x0e,0x38,0x2a,0x54,0x46,0x70,0x62,0x8c,0x9e,0xa8,0xba,0xc4,0xd6,0xe0,0xf2
